#!/usr/bin/perl -w
use strict;

# This is not a perl module; it's 'require'd.
# It contains the world data.

use vars qw(%intros %parser_msgs %rooms %objects %rules);

BEGIN
{
  # entries in the table are theme, world of that theme, starting location
  # for that theme
  my %table = (
    '_' => [ 'zilch', 'zilch', 10, [],
             [ 'Tedium of', 'Incredible', 'Annoyance' ] ],
    'a' => 'j',
    'b' => 'y',
    'c' => [ 'cloak', 'city', 10, [1], 
             [ 'Cloak of', 'Utter', 'Darkness' ] ],
    'd' => [ 'hunter', 'cave', 10, [1, 2],
             [ 'Hunter in the', '', 'Darkness' ] ],
    'e' => [ 'advent', 'forest', 12, [],
             [ 'The Exploration of', 'Colossal', 'Cave' ] ],
    'f' => 's',
    'g' => [ 'galatea', 'city', 18, [],
             [ 'Galatea vs the', 'Unimplemented', 'Parser' ] ],
    # h reserved for hq9+
    'i' => 'v', # [ 'sf', 'princess', 10 ],   # ROOMS/OBJS shared w/xxx
    'j' => [ 'goldskull', 'forest', 10, [],
             [ 'The Quest for the', 'Golden', 'Skull' ] ],
    'k' => 'n',
    'l' => [ 'grip', 'ruins', 15, [],
             [ 'Analysis of the', 'Inner', 'Self' ] ],
    'm' => 'r',
    'n' => [ 'anchorhead', 'city', 14, [],
             [ 'The Horror of', 'Elder', 'Gods' ] ],
    'o' => [ 'office', 'normal', 14, [],
             [ 'The Art of', 'Modern', 'Warfare' ] ],
    'p' => [ 'pirate', 'water', 16, [8],
             [ 'X Marks the', '', 'Spot'] ],
    # q reserved for hq9+
    'r' => [ 'ruins', 'ruins', 10, [],
             [ 'Tortillas in the ', '', 'Mist' ] ],
    's' => [ 'scott', 'water', 10, [],
             [ 'Beware of', 'Sleeping', 'Dragons' ] ],
    't' => 'p',
    'u' => 'l',
    'v' => [ 'spy', 'cave', 15, [5],
             [ 'Operation:', '', 'Unlikely' ] ],
    'w' => 'c',
    'x' => 'g', # [ 'xxx', 'princess', 10 ], # ROOMS/OBJS shared with sf
    'y' => [ 'house', 'normal', 10, [],
             [ 'My', 'First', 'Adventure' ] ],
    'z' => 'e', 
  );

  foreach my $c ('a'..'z')
  {
    next unless exists $table{$c};
    $table{$c} = $table{$table{$c}} unless ref($table{$c});
  }

  sub get_theme
  {
    my $c = shift;
    halt("Unknown char->theme '$c'\n") unless exists $table{$c};
    return $table{$c}->[0];
  }

  sub get_world 
  {
    my $c = shift;
    halt("Unknown char->world '$c'\n") unless exists $table{$c};
    confess "Oops: $c $table{$c}\n" unless ref $table{$c};
    return $table{$c}->[1];
  }

  sub get_start_room
  {
    my $c = shift;
    halt("Unknown char->start_room '$c'\n") unless exists $table{$c};
    return $table{$c}->[2];
  }

  sub get_inventory
  {
    my $c = shift;
    halt("Unknown char->inv '$c'\n") unless exists $table{$c};
    return $table{$c}->[3];
  }

  sub get_title
  { 
    my $c = shift;
    halt("Unknown char->title '$c'\n") unless exists $table{$c};
    return $table{$c}->[4];
  }
}

%intros = (
  zilch => "Welcome to Annoyotron!",
  goldskull =>
     "Treasure! Adventure! Lost civilization! All this and more awaits, " .
     "if you can survive the traps surrounding ...\nTHE GOLD SKULL",
  advent => 
     "Welcome to Adventure!",
  cloak =>
     "Hurrying through the rainswept November night, you're glad to see " .
     "the bright lights of the Opera House. It's surprising that there " .
     "aren't more people about but, hey, what do you expect in a cheap " .
     "demo game...?",
  ruins =>
     "Days of searching, days of thirsty hacking through the briars of " .
     "the forest, but at last your patience was rewarded. A discovery!",
  grip =>
     "Rain and mud.\n\nThose are your first solid memories. Rain pouring " .
     "down on your head, filling your eyes. Mud beneath your feet, filling " .
     "your shoes. Other details slowly filter in. The trees surrounding " .
     "you. The leaden skies above. The chill wind cutting through your " .
     "clothes with ease.\n\nShelter would be a good beginning.",
  scott => 
     "A voice booms out ...\nTreasures have an * in their name. Say 'SCORE' " .
     "If you need a hint on something, try 'HELP'.",
  hunter =>
     "Nearly -- nearly. The animal stink is rank and close. You raise your " .
     "crossbow, try to peer beyond dark, wet stone.",
  anchorhead =>
     "You take a deep breath of salty air as the first raindrops begin to " .
     "spatter the pavement, and the swollen, slate-colored clouds that " .
     "blanket the sky mutter ominous portents amongst themselves over the " .
     "little coastal town of Anchorhead. You're to meet with the real " .
     "estate agent and pick up the keys to your new house while your " .
     "husband Michael runs across town to take care of some paperwork at " .
     "the university. He'll be back to pick you up in a few minutes, and " .
     "then the two of you can begin the long, precarious process of " .
     "settling in.",
   house => 
     "Yawn. You are awakened by the bright morning light streaming in past " .
     "the blinds in your bedroom. Another day as an unemployed but " .
     "aspiring IF author. Maybe today's the big day that you'll write some " .
     "code!",
   office =>
     "Skillfully you fill in the last annual-depreciation calculation and " .
     "then print the report out on the printer by your desk. In your old " .
     "office they knew enough not to waste your time on this penny-ante " .
     "stuff, but in this department nobody knows your name. Yet. Once you " .
     "hand in this report, you'll be well on your way to -- You freeze. " .
     "Your gaze locks on your empty desk drawer. The paperclips are gone. " .
     "That bastard Raymond must have been at your drawer again.\n\nYou " .
     "toss the report aside as you stand up from your Aeron (a little " .
     "going-away gift from the boys). The report can wait: you're got " .
     "a score to settle and paperclips to find.",
   galatea =>
     "You come around a corner, away from the noise of the opening.\n\n" .
     "There is only one exhibit.  She stands in the spotlight, with her " .
     "back to you: a sweep of pale hair on paler skin, a column of emerald " .
     "silk that ends in a pool at her feet.  She might be the model in a " .
     "perfume ad; the trophy wife at a formal gathering; one of the guests " .
     "at this very opening, standing on an empty pedestal in some ironic " .
     "act of artistic deconstruction--\n\n" .
     "You hesitate, about to turn away.  Her hand balls into a fist.\n\n" .
     "\"They told me you were coming.\"",
   pirate =>
     "Yarr, it's a pirate's life for you. Specifically, it's that special " .
     "point in a pirate's life when it's time to bury some treasure. You " .
     "just need to find the right island ...",
   spy =>
     "The local contact dropped you off on the street corner. Said she'd " .
     "be back at 0300. An hour should be plenty to get in and out -- " .
     "if it takes any longer the mission's gotten out of control. " .
     "So you've got to keep things under control and get a copy of " .
     "that data. ",
);

%parser_msgs = (
  zilch => 
  {
    'quit' => "What, giving up already?",
    'pcdesc' => "You look faintly annoyed.",
  },
  goldskull =>
  {
    'pcdesc' => "You look about the same as always.",
    'score' => "The gold skull is the biggest score you'll ever make .. " .
               "the only question is how to get it.",
    'win' => "\n*** Nevertheless, you've won ***",
  },
  advent =>
  {
    'pcdesc' => "You look about the same as always.",
    'score' => "You have scored some points in some turns, earning you the " .
               "rank of Adventurer.",
  },
  cloak => 
  {
    'pcdesc' => "Damp but dashing.",
    'darkroom' => "The darkness doubtlessly hides any number of things you " .
                  "might accidentally disturb. ",
    'score' => "Demo games don't work that way.",
    'die' => "\n*** You have lost ***",
  },
  ruins => 
  {
    'pcdesc' => "As good-looking as ever.",
    'darkroom' => "The darkness of ages presses in on you, and you feel " .
                  "claustrophobic.\n\nSomewhere nearby, tiny claws are " .
                  "scuttling.",
    'score' => "Demo games don't work that way.",
  },
  grip => 
  {
    'pcdesc' => "What you can see of yourself is familiar.",
    'score' => "Surreal games of inner exploration don't work that way.",
    'win' => "\n*** You have successfully changed your addiction from " .
             "cigarettes to caffeine ***",
  },
  scott =>
  {
    'pcdesc' => "You are a bold adventurer!",
    'score' => "Your score: 0 out of 100.",
  },
  hunter => 
  {
    'pcdesc' => "You're tired and covered with mud.",
    'score' => "You: zero. The Wumpus: zero.",
    'win' => "\n*** Well, that was different ***",
  },
  anchorhead =>
  {
    'pcdesc' => "You look good, considering. A bit rattled by recent " .
        "events, perhaps, but well-poised and collected, all the same.",
    'win' => "\n*** The story advances ***",
    'die' => "\n*** The Elder Gods are triumphant ***",
  },
  house => 
  {
    'pcdesc' => "You look like an unemployed but aspiring IF programmer " .
        "of average height and weight and slobbiness.",
  },
  office => 
  {
    'pcdesc' => "You are a student of the way of fist and filofax.",
    'noviolence' => "Violence solves many things, but only when applied " .
                    "precisely.", 
  },
  galatea => 
  {
    'pcdesc' => "You see nothing special about yourself.",
  },
  pirate =>
  {
    'pcdesc' => "You're a pirate: pegleg, artificial nose, the works.",
    'score' => "That's one of the downsides to being at sea for months.",
    'win' => "\n*** Time for some grog ***",
  },
  spy =>
  {
    'pcdesc' => "You are dressed as inconspicuously as possible.",
    'win' => "\n*** Mission accomplished ***",
    'die' => "\n*** The operation has been compromised ***",
  },
);

%rooms = (
  zilch =>
  [
    [ "Treasure Room",
      "You are in a small room, with exits north, east, and west. " .
      "The walls are hung with curtains, and in the center of the " .
      "room is a glass case on a platform studded with levers. " . 
      "Inside the glass case are a small jade statuette and a large " .
      "ruby. The red lever is depressed. The blue lever is raised. " .
      "The black lever is pulled to the left. The white lever is " .
      "pulled to the right and raised. The gold lever is angled " . 
      "slightly to the northeast and rotated twelve degrees.\n\n" .
      "A Suspicious Gurvil regards you silently.", {}, [] ],
    [ "Magical Unicorn Fairyland", 
      "Because you cheated and edited the savefile, you are denied the " .
      "pleasure of viewing Magical Unicorn Fairyland.", {}, [] ],
    [ "Magical Unicorn Fairyland", 
      "Because you cheated and edited the savefile, you are denied the " .
      "pleasure of viewing Magical Unicorn Fairyland.", {}, [] ],
    [ "Magical Unicorn Fairyland", 
      "Because you cheated and edited the savefile, you are denied the " .
      "pleasure of viewing Magical Unicorn Fairyland.", {}, [] ],
    [ "Magical Unicorn Fairyland", 
      "Because you cheated and edited the savefile, you are denied the " .
      "pleasure of viewing Magical Unicorn Fairyland.", {}, [] ],
    [ "Magical Unicorn Fairyland", 
      "Because you cheated and edited the savefile, you are denied the " .
      "pleasure of viewing Magical Unicorn Fairyland.", {}, [] ],
    [ "Magical Unicorn Fairyland", 
      "Because you cheated and edited the savefile, you are denied the " .
      "pleasure of viewing Magical Unicorn Fairyland.", {}, [] ],
    [ "Magical Unicorn Fairyland", 
      "Because you cheated and edited the savefile, you are denied the " .
      "pleasure of viewing Magical Unicorn Fairyland.", {}, [] ],
    [ "Magical Unicorn Fairyland", 
      "Because you cheated and edited the savefile, you are denied the " .
      "pleasure of viewing Magical Unicorn Fairyland.", {}, [] ],
  ],
  forest =>
  [ 
    [ "Outside Cave",
      "You're standing in bright sunlight just outside of a large, dark, " .
      "foreboding cave, which lies to the north.  To the south a winding " .
      "trail leads back to civilization.",
      { north => 11 }, [] ],
    [ "Cave",
      "You're inside a dark, damp cave.  Sunlight pours in from a passage " .
      "to the south.  Some distance from the entrance you notice a large " .
      "stone pedestal, and there's a big sign on the wall.",
      { south => 10 }, [] ],
    [ "At End Of Road",
      "You are standing at the end of a road before a small brick building. " .
      "Around you is a forest. A small stream flows out of the building and " .
      "down a gully.",
      { north => 15, south => 14, west => 15, east => 13, in => 13 }, [] ],
    [ "Inside Building",
      "You are inside a building, a well house for a large spring.",
      { west => 12, out => 12 }, [] ],
    [ "In A Valley",
      "You are in a valley in the forest beside a stream tumbling along a " .
      "rocky bed. At your feet all the water of the stream splashes into a " .
      "2-inch slit in the rock. Downstream the streambed is bare rock.",
      { north => 12, south => 16, west => 15, east => 15 }, [] ],
    [ "In Forest",
      "You are in open forest near both a valley and a road.",
      { north => 12, south => 15, west => 14, east => 14 }, [] ],
    [ "Outside Grate", 
      "You are in a 20-foot depression floored with bare dirt. Set into the " .
      "dirt is a strong steel grate mounted in concrete. A dry streambed " .
      "leads into the depression.",
      { north => 14, south => 15, west => 15, east => 15, down => 17 }, [] ],
    [ "Below the Grate",
      "You are in a small chamber beneath a 3x3 steel grate to the surface. " .
      "A low crawl over cobbles leads inward to the west.",
      { west => 18, up => 16 }, [] ],
    [ "In Debris Room",
      "You are in a debris room filled with stuff washed in from the " .
      "surface. A low wide passage with cobbles becomes plugged with mud " .
      "and debris here, but an awkward canyon leads upward and west.",
      { east => 17 }, [DARK] ],
  ],
  city =>
  [
    [ "Foyer of the Opera House", 
      "You are standing in a spacious hall, splendidly decorated in red and " .
      "gold, with glittering chandeliers overhead. The entrance from the " .
      "street is to the north, and there are doorways south and west.",
      { west => 11, south => 12 }, [] ],
    [ "Cloakroom", 
      "The walls of this small room were clearly once lined with hooks, " .
      "though now only one remains. The exit is a door to the east.",
      { east => 10 }, [] ],
    [ "Foyer Bar",
      "The bar, much rougher than you'd have guessed after the opulence of " .
      "the foyer to the north, is completely empty. There seems to be some " .
      "sort of message scrawled in the sawdust on the floor.", 
      { north => 10 }, [DARK] ],
    [ "Narrow Street",
      "As the lane winds along from east to west, it narrows until the " .
      "steep, jagged rooftops on either side of the street practically " .
      "touch each other. To the south, a side street leads across " .
      "Whateley Bridge toward the center of town, and a twisting lane " .
      "leads up a hill to the northwest. A short flight of steps to the " .
      "north leads down to the local watering hole. ", { east => 14 }, [] ],
    [ "Outside the Real Estate Office",
      "A grim little cul-de-sac, tucked away in a corner of the " .
      "claustrophobic tangle of narrow, twisting avenues that largely " .
      "constitute the older portion of Anchorhead. Like most of the streets " .
      "in this city, it is ancient, shadowy, and leads essentially nowhere. " .
      "The lane ends here at the real estate agent's office, which lies to " .
      "the east, and winds its way back toward the center of town to the " .
      "west. A narrow, garbage-choked alley opens to the southeast. ",
      { west => 13, southeast => 15 }, [] ],
    [ "Alley",
      "This narrow aperture between two buildings is nearly blocked with " .
      "piles of rotting cardboard boxes and overstuffed garbage cans. Ugly, " .
      "half-crumbling brick walls to either side totter oppressively over " .
      "you. The alley ends here at a tall, wooden fence.\n\nHigh up on the " .
      "wall of the northern building there is a narrow, transom-style " .
      "window. ", { northwest => 14 }, [] ],
    [ "File Room",
      "Peering through the murk, you can make out the blocky outlines of " .
      "filing cabinets lining the walls and a doorway to the west. A window " .
      "high up on the south wall lets in a very faint illumination. ",
      { west => 17, south => 15, out => 15 }, [] ],
    [ "Office",
      "Pallid gray light trickles in through the drawn blinds. The office " .
      "is deserted, papers still scattered across the top of the desk. The " .
      "front door lies west, and the file room lies east.", 
      { east => 16 }, [] ],
    [ "The Gallery's End",
      "Unlit, except for the single spotlight; unfurnished, except for " .
      "the defining swath of black velvet.  And a placard on a little " .
      "stand.\n\nOn the pedestal is Galatea.", {}, [] ],
  ],
  ruins =>
  [
    [ "\"Great Plaza\"",
      "Or so your notes call this low escarpment of limestone, but the " .
      "rainforest has claimed it back. Dark olive trees crowd in on all " .
      "sides, the air steams with the mist of a warm recent rain, midges " .
      "hang in the air. \"Structure 10\" is a shambles of masonry which " .
      "might once have been a burial pyramid, and little survives except " .
      "stone-cut steps leading down into darkness below.", 
      { down => 11, in => 11 }, [] ],
    [ "Square Chamber",
      "A sunken, gloomy stone chamber, ten yards across. A shaft of " .
      "sunlight cuts in from the steps above, giving the chamber a diffuse " .
      "light, but in the shadows low lintelled doorways to east and south " .
      "lead into the deeper darkness of the Temple.\n\nCarved inscriptions " .
      "crowd the walls, floor and ceiling.", 
      { up => 10, east => 12, south => 13 }, [] ],
    [ "Wormcast",
      "A disturbed place of hollows carved like a spider's web, strands " .
      "of empty space hanging in stone. The only burrows wide enough to " .
      "crawl through begin by running northeast, south and upwards.",
      { west => 11 }, [DARK] ],
    [ "Stooped Corridor",
      "A low, square-cut corridor, running north to south, stooping you " .
      "over. The south passage ends with a massive door of yellow stone.",
      { north => 11, south => 14 }, [DARK] ],
    [ "Shrine",
      "This magnificent Shrine shows signs of being hollowed out from " .
      "already-existing limestone caves, especially in the western of the " .
      "two long eaves to the south.\n\nVividly busy paintings, of the " .
      "armoured Lord trampling on a captive, are almost too bright to look " .
      "at, the graffiti of an organised mob.",
      { north => 13, southeast => 12 }, [] ],
    [ "Muddy Glade",
      "The trees around you are spaced far enough to provide little " .
      "shelter, allowing the stinging rain to pelt you. A grey mist " .
      "conceals everything past four meters.", {}, [] ],
    [ "Muddy Field",
      "Once the field might have been covered in grass. Now the grass is " .
      "but a memory. Mud covers the ground in its place, fed by the " .
      "constant rain.", {}, [] ],
    [ "Front of Building",
      "Looming suddenly out of the gloom is a large marble building. " .
      "Its columns jut skyward, raked by the unending rain. To the north " .
      "lies its entrance.", { north => 18, in => 18 }, [] ],
    [ "Foyer",
      "The two-story height of the foyer is channelled into a narrower " .
      "hallway to the north. On the north wall to the west of the hallway's " .
      "beginning is an open doorway. The building's entrance to the south " .
      "is reflected in the marble floor.", {}, [] ],
  ],
  water =>
  [
    [ "Meadow", 
      "You're in a sunny meadow. A dragon is sleeping here!\n\nA sign reads " .
      "\"In some cases mud is good, in others...\"",
      { north => 11, east => 11, south => 13, west => 10 }, [] ],
    [ "Lakeside", "You're on the shore of a lake.",
      { north => 12, east => 13, south => 11, west => 10, down => 12 }, [] ],
    [ "Bog", "You're in a quick-sand bog.", {}, [] ],
    [ "Dismal Swamp", "You're in a dismal swamp. There is a cypress tree " .
      "here.",
      { north => 10, east => 13, south => 13, west => 14, in => 15, 
        down => 15 }, [] ],
    [ "Grove", "You're in a hidden grove.", { east => 13 }, [] ],
    [ "Inside Stump", 
      "You're in a large hollow damp stump in the swamp.\n\nA sign reads \"" .
      "Leave treasure here - (say 'SCORE')\"", 
      { up => 13 }, [] ],
    [ "Aboard the Good Ship Squidbeak", 
      "This is a fine ship and, since you're the captain, she's all " .
      "yours. It sails over the bounding brine like nobody's business, " .
      "and what's more, there's a pretty nifty crow's nest.", 
      { up => 17, north => 18 }, [] ],
    [ "Up in the Crow's Nest", 
      "Up in the crow's nest you can watch the sea for miles around, " .
      "count your treasure, or take a quick nap without the crew " .
      "seeing. ", { down => 16 }, [] ],
    [ "Skull Isle", 
      "\"Skull Isle\" might not actually be the name of this place, but " .
      "pirates were never known for original cartography. It has a nice " .
      "stretch of sandy beach, some palm trees, and probably a monkey " .
      "somewhere.", { south => 16 }, [] ],
  ],
  cave =>
  [
    [ "Branching Cave",
      "The cave narrows ahead, a tight crawl winding around folds of " .
      "limestone. A low passage descends to your right, and there is a " .
      "side chamber on the left.", {}, [] ],
    [ "Cavern", "This is a cave (BUG)", {}, [] ],
    [ "Cavern", "This is a cave (BUG)", {}, [] ],
    [ "Canyon Wall",
      "Massive buttresses of stone rise all around you, climbing the canyon " .
      "wall, flowing down again in cascades of pale fluorite. A path seems " .
      "to climb up among them.", { up => 14, out => 14 }, [] ],
    [ "Outside",
      "The night sky is retreating before the advance of the rising sun. " .
      "Already the desert is becoming hot -- there's no telling when the " .
      "natives will wake.\n\nA strange beast, perhaps a native " .
      "riding-animal, is tethered nearby.", {}, [] ],
    [ "Back Alley", 
      "An ordinary back alley, out of sight of the street and not of " .
      "interest. The perfect place to put a backup data center, they must " .
      "have thought. If you do things right, they'll keep thinking that " .
      "until it's too late. There's a metal door in the south wall, and an " .
      "entrance to a disused warehouse to the east.", 
      { south => 16, east => 17 }, [] ],
    [ "Storage", 
      "The room is dark and empty except for dust. Probably abandoned, " .
      "although the locked door suggests it must have had use at some " .
      "point.", { north => 15 }, [ DARK ] ],
    [ "Warehouse", 
      "Not as abandoned as it looks from outside; a glint of light of glass " .
      "reveals the security camera high in the corner. On the far wall " .
      "there's a swipecard reader. ", 
      { west => 15, down => 18 }, [ DARK ] ],
    [ "Data Center", 
      "The data center is as small as the storage room, but it's full: " .
      "generators hum faintly in time with the computer's fan. In the " .
      "ceiling a ladder leads up to the trapdoor.", 
      { up => 17 }, [ DARK ] ],
  ],
  normal =>
  [
    [ "Your Bedroom",
      "Sunlight streams in through the blinds in your bedroom. The room " .
      "is kinda messy and you haven't made your bed, but hey, it's home. " .
      "The living room is to the west.", { west => 11 }, [] ],
    [ "Your Living Room",
      "A tv and a couch are the main pieces of furniture in your living " .
      "room. It could maybe be cleaned up a bit, but that's the bachelor " .
      "lifestyle for you! Your bedroom is back east, the den with your " .
      "computer is to the north, and the kitchen is to the west.", 
      { north => 13, east => 10, west => 12 }, [] ],
    [ "Your Kitchen",
      "The kitchen doesn't get much use because you mostly order pizza. In " .
      "fact, you don't have a fridge or any difficult-to-code appliances " .
      "at all. Pretty much just the one cabinet and that's it. The living " .
      "room is back east.", { east => 11 }, [] ],
    [ "Your Den",
      "Ah, the den -- it could stand to be picked up a bit, but you've " .
      "been so busy scribbling notes for your IF game on pieces of paper " .
      "you haven't had time! The living room is back south.\n\nYour " .
      "computer sits there enticingly.", { south => 11 }, [] ],
    [ "Your Cubicle",
      "Your cubicle is sparse. Spare. Stripped for action. Just your pc, " .
      "a couple of peripherals, and your burning rage. The exit from the " .
      "cube lies towards the rising sun.", { east => 15 }, [] ],
    [ "Raymond's Cubicle",
      "The cubicle of the man who crossed you. Your own cubicle is to the " .
      "west, your boss's office is to the east, and there are more cubicles " .
      "and the supply room to the north. ",
      { west => 14, east => 16, north => 17 }, [] ],
    [ "Mr. McNeeble's Office",
      "The leader of this whole operation has his base of operations here. " .
      "You have to admire his tactical positioning -- the window lets him " .
      "keep an eagle eye on the supply room to the north, while the doorway " .
      "to the west lets him make a quick flanking maneuver if and when it " .
      "becomes necessary.", { west => 15 }, [] ],
    [ "Abandoned Cubicle",
      "It's pitiful -- this department is dying, anyone can see it. So they " .
      "transfer you here. You survey the desolate cubicle with a survivor's " .
      "eye. Not much -- and it appears all the sparser next to the riches " .
      "the supply room to the east. ",
      { east => 18, south => 15 }, [] ],
    [ "Supply Room",
      "Ah, the supply room. You've thought about it before, even made the " .
      "occasional supervised trip, for rubber bands, #2 pencils, or " .
      "carefully-rationed quantities of staples. But this is the first time " .
      "you're here alone, and you've only got one thing on your mind. Well, " .
      "two, if you count revenge. The only exit is west, although there's a " .
      "window to your boss's office to the south.", { west => 17 }, [] ],
  ],
);

%objects = (
  zilch => 
  [
    [ "sparkly widget", "placeholder", NOWHERE, [TREASURE] ],
    [ "interesting thingy", "placeholder", NOWHERE, [TREASURE] ],
    [ "exciting object", "placeholder", NOWHERE, [TREASURE] ],
    [ "captivating artifact", "placeholder", NOWHERE, [TREASURE] ],
    [ "noticeable doodad", "placeholder", NOWHERE, [TREASURE] ],
    [ "amusing gadget", "placeholder", NOWHERE, [TREASURE] ],
    [ "pleasant mechanism", "placeholder", NOWHERE, [TREASURE] ],
    [ "handy gizmo", "placeholder", NOWHERE, [TREASURE] ],
    [ "excellent device", "placeholder", NOWHERE, [TREASURE] ],
  ],
  forest =>
  [
    [ "small rock", "It looks like an ordinary small rock to me.", 10, [] ],
    [ "stone pedestal", "A flat stone pedestal.", 11, [FIXED, SURFACE] ],
    [ "gold skull", 
      "It's a small skull, the size of your clenched fist and cast from " .
      "solid gold. It grins back at you, undoubtedly well aware of its " .
      "priceless value.", 2, [TREASURE] ],
    [ "set of keys", "It's just a normal-looking set of keys.", 13, [KEY] ],
    [ "tasty food", "Sure looks yummy!", 13, [] ],
    [ "shiny brass lamp", "It is a shiny brass lamp.", 13, 
      [SWITCHABLE, LIGHT] ],
    [ "grate mounted in concrete", 
      "It just looks like an ordinary grate mounted in concrete.", 16,
      [FIXED, LOCKABLE, OPENABLE] ],
    [ "wicker cage", "It's a small wicker cage.", 17, [OPENABLE] ],
    [ "black rod", "It's a three foot black rod with a rusty star on an end.",
      18, [TREASURE] ],
  ],
  city =>
  [
    [ "velvet cloak", 
      "A handsome cloak, of velvet trimmed with satin, and slightly " .
      "spattered with raindrops. Its blackness is so deep that it almost " .
      "seems to suck light from the room.", 10, [WEARABLE] ],
    [ "anticloak", "A brightly-lit anticloak.", NOWHERE, [HIDDEN, LIGHT] ],
    [ "brass hook", "It's just a small brass hook, screwed to the wall.",
      11, [FIXED, SURFACE] ],
    [ "message in the sawdust", "The message is unreadable.", 12, [FIXED] ],
    [ "garbage can", 
      "The metal garbage cans are stuffed to overflowing with slowly " .
      "decomposing refuse.", 15, [FIXED] ],
    [ "office window", 
      "It's about eighteen inches wide and a foot tall, with hinges along " .
      "the top that allow it to swing out.", 15, [FIXED, OPENABLE] ],
    [ "paper-covered desk", 
      "The top of the desk is littered with paperwork -- notes, messages, " .
      "files, etc.; the way most desks tend to get at the end of a busy " .
      "day. ", 17, [FIXED, SURFACE] ],
    [ "telephone and answering machine", 
      "A simple combination telephone and answering machine, with a small " .
      "display indicating messages received and a button labeled \"PLAY\".", 
      7, [] ],
    [ "ring of keys", 
      "It's just an ordinary-looking keyring. There are two keys (a key to " .
      "the house and a key to the cellar) on it.", NOWHERE, [KEY] ],
  ],
  ruins =>
  [
    [ "packing case", 
      "Your packing case rests here, ready to hold any important cultural " .
      "finds you might make, for shipping back to civilisation.",
      10, [FIXED, CONTAINER, OPENABLE] ],
    [ "speckled mushroom", 
      "The mushroom is capped with blotches, and you aren't at all sure " .
      "it's not a toadstool.",
      10, [] ],
    [ "sodium lamp", "It is a heavy-duty archaeologist's lamp.", 10, 
      [SWITCHABLE, LIGHT] ],
    [ "glistening white eggsac", 
      "A glistening white eggsac, like a clump of frogspawn the size of a " .
      "beach ball.", 12, [] ],
    [ "stone key", "You see nothing special about the stone key.", NOWHERE,
      [KEY] ],
    [ "massive stone door", "It's just a big stone door.", 13, 
      [FIXED, OPENABLE, LOCKABLE ] ],
    [ "jade mosaic face mask", "How exquisite it would look in the Museum.",
      14, [TREASURE] ],
    [ "human head", 
      "Other than being buried in mud to his chin, the head is reassuringly " .
      "normal. His hair and mustache are a matted brown, perhaps from " .
      "genetics, perhaps from mud. Rivulets of water run down his creased " .
      "face.", 16, [FIXED, ACTOR] ],
    [ "can of Coke", "The familiar red-and-white can is a comforting sight.", 
      18, [TREASURE] ],
  ],
  water =>
  [ 
    [ "sleeping Dragon", "The dragon is sleeping!", 10, [FIXED, ACTOR] ],
    [ "rusty axe (BUNYON)", "The magic word BUNYON is written on the haft.",
      11, [] ],
    [ "small statue of a blue ox", "A small blue ox! It looks valuable!",
      12, [TREASURE] ],
    [ "evil smelling mud", "The mud smells bad!", 13, [] ],
    [ "cypress tree", "Someone should chop it down!", 13, [FIXED] ],
    [ "treasure chest", 
      "The chest is filled with gold, jewels, and chitlins.", 16, 
      [TREASURE, CONTAINER, OPENABLE] ],
    [ "crew of swabbies", 
      "You call them the Tigers, but they should really be called the " .
      "Q-Tips, because -- nah. ", 16, [ACTOR, FIXED] ],
    [ "spyglass", "(That's pirate-talk for \"telescope\".)", 17, [] ],
    [ "shovel", "Best get shovellin'.", NOWHERE, [] ],
  ],  
  cave =>
  [
    [ "crossbow", 
      "The crossbow is well-designed enough to be simple. Laminated " .
      "leaf-laths, fiber string, and a cocking lever that runs the full " .
      "length of the stock.", 10, [] ],
    [ "water skin", "Old, tanned leather. And, unfortunately, empty.",
      10, [] ],
    [ "wumpus", 
      "Great sucker feet to move adeptly through caves. Larger-than-human " .
      "eyes to see in the darkness. Claws to rend and tear.", 10, 
      [ACTOR, FIXED] ],
    [ "strange beast", 
      "A peculiar beast -- like a horse, in a way, but covered in fur like " .
      "a dog. Strangest of all, its back rises like a small hill. Still, " .
      "you suppose you could ride it: it must be faster than going on foot, " .
      "wounded as you are.", 14, [ACTOR, FIXED] ],
    [ "multipurpose spy device", 
      "After years of issuing spy-cameras and spy-picklocks and spy-shavers " .
      "HQ finally wised up and made a single gadget.", NOWHERE, 
      [SWITCHABLE, LIGHT, KEY, TREASURE, WEARABLE] ],
    [ "metal door", "The metal door looks old but solid.", 15, 
      [FIXED, OPENABLE, LOCKABLE] ],
    [ "id card", 
       "A simple swipecard with an id number and a bar code.", 
       16, [KEY] ],
    [ "security camera", 
      "The security camera has been concealed in the corner. Not well enough.",
      17, [FIXED] ],
    [ "computer", 
      "This is it. The files must be here. There's a plug in the side.", 
      18, [FIXED] ],
  ],
  normal =>
  [
    [ "tv", "The television is great!", 11, [SWITCHABLE, FIXED] ],
    [ "computer", 
      "This is the computer that you (plan to) write all your IF " .
      "masterpieces on, just like that Plotkin guy. Yessirree. At any " .
      "moment you could just sit down and write some IF. Any moment.", 13,
      [SWITCHABLE, FIXED] ],
    [ "cabinet", "What mysteries lie within the kitchen cabinet?", 12,
      [ CONTAINER, OPENABLE, FIXED ] ],
    [ "bag of cookies", "Yeeeeeeah cookies!", 3, [TREASURE] ],
    [ "Raymond", 
      "You have nothing more to say about Raymond. His crime is clear, and " .
      "and punishment comes swiftly to the unvirtuous man.", 15, 
      [ACTOR, FIXED] ],
    [ "Mr. McNeeble",
      "He may look weak, and his mind unfocused, but his grasp on power " .
      "is tight.", 16, [ACTOR, FIXED] ],
    [ "report",
      "The report is covered with a large number of handwritten " .
      "corrections and annotations, and marked URGENT across the top.",
      15, [] ],
    [ "paper shredder",
      "A portable high-powered paper shredder. A true warrior's tool.",
      17, [CONTAINER, SWITCHABLE] ],
    [ "box of paperclips",
      "A box of paperclips. Simple in appearance, just as an arrow's flight " .
      "is simple. And just as sure.", 18, [TREASURE] ],
  ],
);  

%rules = (
  zilch => [ [ [ 0, \&annoying_parser ] ], [], [] ],
  forest =>
  [
    [ [ 2, \&leave_cave_with_treasure ],
      [ 2, \&leave_cave_without_treasure ],
      [ 2, \&examine_sign ],
      [ 3, \&pedestal_trap ] ],
    [ [ 2, \&rub_lamp ],
      [ 2, \&eat_tasty_food ],
      [ 2, \&down_through_grate ] ],
    [ [ 2, \&xyzzy_in_cave ],
      [ 2, \&xyzzy_in_well_house ],
      [ 2, \&xyzzy_elsewhere ],
      [ 2, \&wave_rod ] ],
  ],
  city =>
  [
    [ [ 2, \&leave_opera_house ],
      [ 1, \&no_drop_cloak ],
      [ 3, \&teleport_anticloak_in ],
      [ 3, \&teleport_anticloak_out ], ],
    [ [ 1, \&scuff_once ],
      [ 1, \&scuff_twice ],
      [ 2, \&read_message_bad ],
      [ 2, \&read_message_good ], 
      [ 2, \&wander_into_anchorhead ],
      [ 2, \&leave_with_keys ],
      [ 2, \&leave_without_keys ],
      [ 2, \&office_is_locked ], ],
    [ [ 2, \&push_can ],
      [ 2, \&push_can_again ],
      [ 2, \&climb_onto_can ],
      [ 2, \&descend_from_can ],
      [ 3, \&implicitly_descend_from_can ],
      [ 2, \&enter_office_window_closed ],
      [ 2, \&enter_office_from_can ],
      [ 2, \&enter_office_from_ground ],
      [ 2, \&disturb_window_from_ground ],
      [ 2, \&play_message ],
      [ 2, \&look_up_verlac ],
      [ 2, \&look_up_anything ],
      [ 2, \&examine_galatea ],
      [ 2, \&talk_to_galatea_about_cheese ],
      [ 2, \&talk_to_galatea ], ],
  ],
  ruins =>
  [
    [ [ 2, \&stash_treasure ], 
      [ 2, \&stash_non_treasure ], 
      [ 2, \&take_mushroom_first ],
      [ 2, \&eat_mushroom ],
      [ 2, \&smell_mist ],
      [ 1, \&ruins_blocked ],
      [ 2, \&read_inscriptions ],
      [ 2, \&wormcast_scramble ],
      [ 2, \&burn_eggsac ],
      [ 1, \&leave_with_eggsac ], ],
    [ [ 2, \&killer_darkness ],
      [ 1, \&stone_door_locked ],
      [ 2, \&examine_paintings ],
      [ 3, \&take_mask ],
      [ 2, \&wear_mask ],
      [ 1, \&shrine_sw ] ],
    [ [ 2, \&move_in_first_muddy_room ], 
      [ 3, \&desc_for_head ], 
      [ 3, \&head_talks_again ], 
      [ 3, \&head_talks ], 
      [ 2, \&kick_head ], 
      [ 2, \&kiss_head ], 
      [ 2, \&take_head ], 
      [ 2, \&dig_head ], 
      [ 2, \&move_in_second_muddy_room_with_head ], 
      [ 2, \&move_in_second_muddy_room ], 
      [ 2, \&move_in_third_muddy_room ], 
      [ 1, \&enter_muddy_building ], 
      [ 2, \&move_in_muddy_building ], 
      [ 2, \&drink_coke ] ],
  ],
  water =>
  [
    [ [ 2, \&wake_dragon ],
      [ 2, \&dont_disturb_dragon ],
      [ 3, \&mud_wakes_dragon ],
      [ 2, \&bunyon_with_axe ],
      [ 2, \&bunyon_otherwise ],
      [ 2, \&swim_in_quicksand_unloaded ],
      [ 2, \&swim_in_quicksand ] ],
    [ [ 2, \&climb_tree ],
      [ 2, \&tree_blocks_stump ],
      [ 2, \&chop_tree ],
      [ 3, \&say_stump ],
      [ 2, \&drop_treasure_here ] ],
    [ [ 3, \&display_swabbies ], 
      [ 2, \&talk_to_swabbies ], 
      [ 2, \&keelhaul_swabbies ],
      [ 2, \&leave_undocked_ship ], 
      [ 3, \&swabbies_leave_docked_ship ],
      [ 2, \&look_through_spyglass ],
      [ 2, \&count_treasure ],
      [ 2, \&bury_treasure ] ],
  ],
  cave =>
  [
    [ [ 3, \&display_wumpus ],
      [ 2, \&shoot_wumpus_first ],
      [ 1, \&move_past_wumpus ],
      [ 2, \&random_cave_desc ],
      [ 3, \&display_blood_trail ],
      [ 2, \&examine_blood ],
      [ 2, \&move_through_cave ],
      [ 1, \&dont_drop_in_cave ],
      [ 1, \&need_crossbow_to_attack ],
      [ 1, \&move_compass_dir ], 
      [ 2, \&move_relative_dir ] ],
    [ [ 2, \&shoot_wumpus_second ],
      [ 3, \&emerge_from_cave ],
      [ 2, \&ride_camel ],
      [ 2, \&leave_alley ],
      [ 2, \&door_blocks_storage ],
      [ 1, \&morph_into_key ],
      [ 1, \&morph_into_light ] ],
    [ [ 2, \&disable_camera ],
      [ 2, \&disable_other_mechanism ],
      [ 2, \&cant_affect_camera ],
      [ 2, \&camera_watches_swipe ],
      [ 2, \&examine_swipe_reader ],
      [ 1, \&put_to_swipe ],
      [ 2, \&swipe_card ],
      [ 2, \&swipe_multi ],
      [ 3, \&display_trapdoor ],
      [ 2, \&trapdoor_blocks_down ],
      [ 2, \&plug_in_computer ],
      [ 3, \&close_trapdoor ] ],
  ],
  normal =>
  [
    [ [ 2, \&make_bed ],
      [ 2, \&watch_tv ],
      [ 2, \&eat_cookies ],
      [ 2, \&write_if ] ],
    [ [ 3, \&display_raymond ],
      [ 2, \&talk_to_raymond ],
      [ 3, \&raymond_talk_on_entrance ],
      [ 3, \&raymond_babble ],
      [ 1, \&pick_up_report_by_raymond ],
      [ 1, \&cant_leave_with_report ] ],
    [ [ 3, \&display_mcneeble ],
      [ 2, \&talk_to_mcneeble ],
      [ 3, \&mcneeble_babble ],
      [ 3, \&shred_report ],
      [ 3, \&shred_other ],
      [ 1, \&mcneeble_blocks_supply ],
      [ 2, \&cant_reenter_cube ],
      [ 3, \&return_with_paperclips ] ],
  ]
);

sub annoying_parser
{
  my ($verb, $prep, $dobj, $iobj, $command) = @_;
  if ($verb ne "quit" && $verb ne "look")
  {
    my @resp = (
      "Sorry, I didn't understand that command.",
      "I don't know the word '%w'.",
      "You can't %v that.",
      "I don't know how to %v anything.",
      "I beg your pardon?",
      "You seem to want to %v something, but I can't see what.",
      "You aren't holding the %o!",
      "I didn't understand the way that finished.",
      "Sorry, I didn't understand that.",
      "Excuse me?",
      "That's not a verb I recognize.",
      "Are you sure?",
      "I don't understand the punctuation \"%p\".",
      "The word \"%o\" refers to too many objects.",
      "I think you left something out in that sentence.",
      "I don't see any %o here.",
      "I don't know what you're referring to with '%w'.",
      "You'll have to be more specific.", 
    );

    my $punct = ($command =~ /([^a-z0-9\s_-])/i) ? $1 : "";
    my @words = split ' ', $command;
    my $obj = (@words > 1) ? $words[int(rand(@words - 1)) + 1] : "";
    my $word = (@words > 0) ? $words[int(rand(@words))] : "";
    my $text = "";
    while ($text eq "")
    {
      my $r = int(rand(@resp));
      if ($resp[$r] =~ /%p/)
      {
        next if $punct eq "";
        $text = $resp[$r];
        $text =~ s/%p/$punct/;
      }
      elsif ($resp[$r] =~ /%o/)
      {
        next if $obj eq "";
        $text = $resp[$r];
        $text =~ s/%o/$obj/;
      }
      elsif ($resp[$r] =~ /%v/)
      {
        next if $verb eq "";
        $text = $resp[$r];
        $text =~ s/%v/$verb/;
      }
      elsif ($resp[$r] =~ /%w/)
      {
        next if $word eq "";
        $text = $resp[$r];
        $text =~ s/%w/$word/;
      }
      else
      {
        $text = $resp[$r];
      }
    }
      
    output_text($text . "\n");

    return ABORT;
  }

  return CONTINUE;
}

sub leave_cave_with_treasure
{
  my ($verb, $prep, $dobj, $iobj) = @_;
  my $START_ROOM = 10;
  if ($verb eq "south" && within(PC_OBJ, $START_ROOM) && 
      any_with_trait(PC_OBJ, TREASURE) != -1)
  {
    my $text = "You trudge back down the trail, " . 
        thename(any_with_trait(PC_OBJ, TREASURE)) . 
        " firmly in hand, and soon make it to civilization next day. Fame! " .
        "Fortune! Riches!\n\nBut the store is out of milk.  Rats.\n";
    output_text($text);
    msg("win");
    halt();
  }

  return CONTINUE;
}

sub leave_cave_without_treasure
{
  my ($verb, $prep, $dobj, $iobj) = @_;
  my $START_ROOM = 10;
  if ($verb eq "south" && within(PC_OBJ, $START_ROOM))
  {
    output_text("You can't leave now! Where's the treasure you came for?\n");
    return ABORT;
  }

  return CONTINUE;
}

sub examine_sign
{
  my ($verb, $prep, $dobj, $iobj, $command, $dobj_str) = @_;
  my $CAVE = 11;
  if ($verb eq "examine" && $dobj_str =~ /\bsign\b/ && within(PC_OBJ, $CAVE))
  {
    output_text("A large sign, carved into the very living rock of the cave " .
                "wall itself. It reads: \"Trespassers Will Be Persecuted " .
                "and Probably Summarily Shot\".\n");
    return SUCCESS;
  }

  return CONTINUE;
}

sub pedestal_trap
{
  my ($verb, $prep, $dobj, $iobj) = @_;
  if (any_with_trait($state{'objects'}->[PC_OBJ], SURFACE) != -1 && 
      contents(any_with_trait($state{'objects'}->[PC_OBJ], SURFACE)) == 0)
  {
    my $text = "As you lift the skull, a volley of poisonous arrows bursts " .
        "out of the walls! You try to dodge, but they take you by surprise!\n";
    output_text($text);
    msg("die");
    halt();
    return ABORT;
  }

  return CONTINUE;
}

sub rub_lamp
{
  my ($verb, $prep, $dobj, $iobj) = @_;
  my $LAMP = 6;
  if ($verb eq "rub" && $dobj eq $LAMP)
  {
    output_text("Rubbing the electric lamp is not particularly rewarding. " .
                "Anyway, nothing exciting happens.\n");
    return SUCCESS;
  }

  return CONTINUE;
}

sub eat_tasty_food
{
  my ($verb, $prep, $dobj, $iobj) = @_;
  my $FOOD = 5;
  if ($verb eq "eat" && $dobj eq $FOOD)
  {
    output_text("Mm, food.\n");
    $state{'objects'}->[$FOOD] = NOWHERE;
    return SUCCESS;
  }

  return CONTINUE;
}

sub down_through_grate
{
  my ($verb, $prep, $dobj, $iobj) = @_;
  my $GRATE = 7;
  if ($verb eq "down" && flagged($GRATE))
  {
    output_text("The grate is locked and bars the way.\n");
    return ABORT;
  }

  return CONTINUE;
}

sub xyzzy_in_cave
{
  my ($verb, $prep, $dobj, $iobj) = @_;
  my $XYZZY_ROOM = 18;
  my $WELL_ROOM = 13;
  if ($verb eq "xyzzy" && within(PC_OBJ, $XYZZY_ROOM))
  {
    $state{'objects'}->[PC_OBJ] = $WELL_ROOM;
    unread("look");
    return SUCCESS;
  }

  return CONTINUE;
}

sub xyzzy_in_well_house
{
  my ($verb, $prep, $dobj, $iobj) = @_;
  my $XYZZY_ROOM = 18;
  my $WELL_ROOM = 13;
  if ($verb eq "xyzzy" && within(PC_OBJ, $WELL_ROOM))
  {
    $state{'objects'}->[PC_OBJ] = $XYZZY_ROOM;
    unread("look");
    return SUCCESS;
  }

  return CONTINUE;
}

sub xyzzy_elsewhere
{
  my ($verb, $prep, $dobj, $iobj) = @_;
  if ($verb eq "xyzzy")
  {
    output_text("Nothing happens.\n");
    return SUCCESS;
  }

  return CONTINUE;
}

sub wave_rod
{
  my ($verb, $prep, $dobj, $iobj) = @_;
  my $ROD = 9;
  if ($verb eq "wave" && $dobj eq $ROD)
  {
    output_text("There is a sparkling light, and a bridge materializes, " .
                "leading you off into a magical world of adventure and " .
                "having axes thrown at you.\n");
    msg("win");
    halt();
    return ABORT;
  }

  return CONTINUE;
}

sub leave_opera_house
{
  my ($verb, $prep, $dobj, $iobj) = @_;
  my $START_ROOM = 10;
  if ($verb eq "north" && within(PC_OBJ, $START_ROOM))
  {
    output_text("You've only just arrived, and besides, the weather outside " .
                "seems to be getting worse.\n");
    return ABORT;
  }

  return CONTINUE;
}

sub no_drop_cloak
{
  my ($verb, $prep, $dobj, $iobj) = @_;
  my $CLOAK = 1;
  my $CLOAK_ROOM = 11;
  if ($verb eq "drop" && $dobj eq $CLOAK && !within(PC_OBJ, $CLOAK_ROOM))
  {
    output_text("Surely the proper place for a cloak is in a cloakroom.\n");
    return ABORT;
  }

  return CONTINUE;
}

# the anticloak-of-darkness lights the room, natch
sub teleport_anticloak_in
{
  my ($verb, $prep, $dobj, $iobj) = @_;
  my $CLOAK = 1;
  my $ANTICLOAK = 2;
  if (!within($CLOAK, PC_OBJ))
  {
    $state{'objects'}->[$ANTICLOAK] = PC_OBJ;
    # fall through to the continue
  }

  return CONTINUE;
}

sub teleport_anticloak_out
{
  my ($verb, $prep, $dobj, $iobj) = @_;
  my $CLOAK = 1;
  my $ANTICLOAK = 2;
  if (within($CLOAK, PC_OBJ))
  {
    $state{'objects'}->[$ANTICLOAK] = NOWHERE;
    # fall through to the continue
  }

  return CONTINUE;
}

sub scuff_once
{
  my ($verb, $prep, $dobj, $iobj) = @_;
  my $DARK_ROOM = 12;
  if ($verb ne "north" && $verb ne "look" && within(PC_OBJ, $DARK_ROOM) && 
      !is_lit($DARK_ROOM) && !flagged(PC_OBJ))
  {
    output_text("In the dark? You could easily disturb something!\n");
    flagged(PC_OBJ, 1);
    return ABORT;
  }

  return CONTINUE;
}

sub scuff_twice
{
  my ($verb, $prep, $dobj, $iobj) = @_;
  my $DARK_ROOM = 12;
  my $MESSAGE = 4;
  if ($verb ne "north" && $verb ne "look" && within(PC_OBJ, $DARK_ROOM) && 
      !is_lit($DARK_ROOM))
  {
    output_text("In the dark? You could easily disturb something!\n");
    flagged($MESSAGE, 1);
    return ABORT;
  }

  return CONTINUE;
}

sub read_message_bad
{
  my ($verb, $prep, $dobj, $iobj) = @_;
  my $MESSAGE = 4;
  if ($verb eq "examine" && $dobj eq $MESSAGE && flagged($MESSAGE))
  {
    output_text("The message has been carelessly trampled, making it " .
                "difficult to read. You can just distinguish the words...\n");
    msg("die");
    halt();
    return ABORT;
  }

  return CONTINUE;
}

sub read_message_good
{
  my ($verb, $prep, $dobj, $iobj) = @_;
  my $MESSAGE = 4;
  if ($verb eq "examine" && $dobj eq $MESSAGE)
  {
    output_text("The message, neatly marked in the sawdust, reads...\n");
    msg("win");
    halt();
    return ABORT;
  }

  return CONTINUE;
}

sub stash_treasure
{
  my ($verb, $prep, $dobj, $iobj) = @_;
  my $CASE = 1;
  if ($verb eq "put" && $prep eq "in" && $iobj eq $CASE && 
      has_trait($dobj, TREASURE))
  {
    output_text("As you carefully pack away " . thename($dobj) . " a " .
                "red-tailed macaw flutters down from the tree-tops, " .
                "feathers heavy in the recent rain, the sound of its " .
                "beating wings almost deafening, stone falling against " .
                "stone... As the skies clear, a crescent moon rises above " .
                "a peaceful jungle. It is the end of March, 1938, and it is " .
                "time to go home.\n");
    msg("win");
    halt();
    return ABORT;
  }

  return CONTINUE;
}

sub stash_non_treasure
{
  my ($verb, $prep, $dobj, $iobj) = @_;
  my $CASE = 1;
  if ($verb eq "put" && $prep eq "in" && $iobj eq $CASE)
  {
    output_text(ucfirst(thename($iobj)) . " is only for storing your " . 
                "archaeological discoveries.\n");
    return ABORT;
  }

  return CONTINUE;
}

sub take_mushroom_first
{
  my ($verb, $prep, $dobj, $iobj) = @_;
  my $MUSHROOM = 2;
  if ($verb eq "take" && $dobj eq $MUSHROOM && !flagged($MUSHROOM))
  {
    output_text("You pick " . thename($dobj) . ", neatly cleaving its thin " .
                "stalk.\n");
    $state{'objects'}->[$MUSHROOM] = PC_OBJ;
    flagged($MUSHROOM, 1);
    return SUCCESS;
  }

  return CONTINUE;
}

sub eat_mushroom
{
  my ($verb, $prep, $dobj, $iobj) = @_;
  my $MUSHROOM = 2;
  if ($verb eq "eat" && $dobj eq $MUSHROOM)
  {
    output_text("You nibble at one corner, unable to trace the source of an " .
                "acrid taste, distracted by the flight of a macaw overhead " .
                "which seems to burst out of the sun, the sound of the " .
                "beating of its wings almost deafening, stone falling " .
                "against stone.\n");
    $state{'objects'}->[$MUSHROOM] = NOWHERE;
    return SUCCESS;
  }

  return CONTINUE;
}

sub smell_mist
{
  my ($verb, $prep, $dobj, $iobj, $command, $dobj_words) = @_;
  my $PLAZA = 10;
  if (($verb eq "smell" || $verb eq "sniff") && $dobj_words =~ /\bmist\b/ &&
      within(PC_OBJ, $PLAZA))
  {
    output_text("The mist has an aroma reminiscent of tortilla.\n");
    return SUCCESS;
  }

  return CONTINUE;
}

sub ruins_blocked
{
  my ($verb, $prep, $dobj, $iobj, $command, $dobj_words) = @_;
  my $PLAZA = 10;
  my $MUSHROOM = 2;
  if ($verb eq "down" && within(PC_OBJ, $PLAZA) && !within($MUSHROOM, NOWHERE))
  {
    output_text("Rubble blocks the way after only a few steps.\n");
    return ABORT;
  }

  return CONTINUE;
}

sub read_inscriptions
{
  my ($verb, $prep, $dobj, $iobj, $command, $dobj_words) = @_;
  my $CHAMBER = 11;
  if ($verb eq "examine" && $dobj_words =~ /\binscriptions?\b/ &&
      within(PC_OBJ, $CHAMBER))
  {
    output_text("Each time you look at the carvings closely, they seem to " .
                "be still. But you have the uneasy feeling when you look " .
                "away that they're scuttling, moving about. Two glyphs are " .
                "prominent: Arrow and Circle.\n");
    return SUCCESS;
  }

  return CONTINUE;
}

sub wormcast_scramble
{
  my ($verb, $prep, $dobj, $iobj, $command, $dobj_str, $iobj_str, $error) = @_;
  my $PLAZA = 10;
  my $CHAMBER = 11;
  my $WORMCAST = 12;
  my $PASSAGE = 13;
  my $EGGSAC = 4;
  if (within(PC_OBJ, $WORMCAST) && ($verb eq "up" || $verb eq "northeast" ||
                                    $verb eq "south"))
  {
    output_text("The wormcast becomes slippery around you, as though your " .
                "body-heat is melting long hardened resins, and you shut " .
                "your eyes tightly as you burrow through darkness.\n\n");
    $state{'objects'}->[PC_OBJ] = ($PLAZA, $CHAMBER, $PASSAGE)[int(rand(3))];
    $state{'objects'}->[PC_OBJ] = $CHAMBER if within($EGGSAC, PC_OBJ);
    handle_look("look", "", -1, -1, $command, "", "", $error);
    return SUCCESS;
  }

  return CONTINUE;
}

sub burn_eggsac
{
  my ($verb, $prep, $dobj, $iobj, $command, $dobj_words, $iobj_words) = @_;
  my $CHAMBER = 11;
  my $EGGSAC = 4;
  my $KEY = 5;
  if ($verb eq "put" && $dobj eq $EGGSAC && within(PC_OBJ, $CHAMBER) &&
      $iobj_words =~ /\b(sun|light|sunlight)\b/)
  {
    output_text("You drop " . thename($EGGSAC) . " into the glare of the " .
                "shaft of sunlight. It bubbles obscenely, distends and then " .
                "bursts into a hundred tiny insects which run in all " .
                "directions into the darkness. Only spatters of slime and " .
                aname($KEY) . " remain on the floor.\n");
    $state{'objects'}->[$EGGSAC] = NOWHERE;
    $state{'objects'}->[$KEY] = $CHAMBER;
    return SUCCESS;
  }

  return CONTINUE;
}

sub leave_with_eggsac
{
  my ($verb, $prep, $dobj, $iobj) = @_;
  my $CHAMBER = 11;
  my $EGGSAC = 4;
  if ($verb eq "up" && within(PC_OBJ, $CHAMBER) && within($EGGSAC, PC_OBJ))
  {
    output_text("The moment that natural light falls upon " . 
                thename($EGGSAC) . ", it bubbles obscenely and distends. " .
                "Before you can throw it away, it bursts into a hundred " .
                "tiny, birth-hungry insects...\n");
    msg("die");
    halt();
    return ABORT;
  }

  return CONTINUE;
}

sub killer_darkness
{
  my ($verb, $prep, $dobj, $iobj) = @_;
  if (!is_lit($state{'objects'}->[PC_OBJ]) && $verb ne "turn on" && 
      $verb ne "north" && $verb ne "west")
  {
    output_text("Suddenly there is a tiny pain, of a hypodermic-sharp fang " .
                "at your calf. Almost at once your limbs go into spasm, " .
                "your shoulders and knee-joints lock, your tongue " .
                "swells...\n");
    msg("die");
    halt();
    return ABORT;
  }

  return CONTINUE;
}

sub stone_door_locked
{
  my ($verb, $prep, $dobj, $iobj) = @_;
  my $PASSAGE = 13;
  my $DOOR = 6;
  if ($verb eq "south" && within(PC_OBJ, $PASSAGE) && flagged($DOOR))
  {
    output_text("The massive stone door bars the way.\n");
    return ABORT;
  }

  return CONTINUE;
}

sub examine_paintings
{
  my ($verb, $prep, $dobj, $iobj, $command, $dobj_words) = @_;
  my $SHRINE = 14;
  if ($verb eq "examine" && $dobj_words =~ /\b(paintings?|graffiti)\b/ &&
      within(PC_OBJ, $SHRINE))
  {
    output_text("The flesh on the bodies is blood-red. The markers of the " .
                "Long Count date the event to 10 baktun 4 katun 0 tun 0 " .
                "uinal 0 kin, the sort of anniversary when one Lord would " .
                "finally decapitate a captured rival who had been ritually " .
                "tortured over a period of some years, in the Balkanised " .
                "insanity of the Maya city states.\n");
    return SUCCESS;
  }

  return CONTINUE;
}

sub take_mask
{
  my ($verb, $prep, $dobj, $iobj) = @_;
  my $MASK = 7;
  if ($verb eq "take" && $dobj eq $MASK)
  {
    output_text("As you lift " . thename($dobj) . ", you think of your " .
                "packing case, ready to store artifacts for shipment to " .
                "the museum.\n");
    # fall through to the continue:
  }

  return CONTINUE;
}

sub wear_mask
{
  my ($verb, $prep, $dobj, $iobj, $command) = @_;
  my $MASK = 7;
  if ($command =~ /^(wear|don)\b/ && $dobj eq $MASK)
  {
    output_text("Looking through the obsidian eyeslits of the mosaic mask, " .
                "a ghostly presence reveals itself: a mummified calendrical " .
                "priest, who teaches you all about the wonders of Inform " .
                "development, even unto the mysteries of ChooseObjects.\n");
    msg("win");
    halt();
    return ABORT;
  }

  return CONTINUE;
}

sub shrine_sw
{
  my ($verb, $prep, $dobj, $iobj) = @_;
  my $SHRINE = 14;
  if ($verb eq "southwest" && within(PC_OBJ, $SHRINE))
  {
    output_text("The eaves taper out into a crevice which would wind " .
                "further if it weren't jammed tight with icicles. The " .
                "glyph of the Crescent is not quite obscured by ice.\n");
    return ABORT;
  }

  return CONTINUE;
}

sub move_in_first_muddy_room
{
  my ($verb, $prep, $dobj, $iobj, $command, $dobj_str, $iobj_str, $error) = @_;
  my $MUDDY_ONE = 15;
  my $MUDDY_TWO = 16;
  if (is_dir($verb) && within(PC_OBJ, $MUDDY_ONE))
  {
    output_text("You strike off in that direction, but soon become " .
                "disoriented by the rain.\n\n");
    $state{'objects'}->[PC_OBJ] = $MUDDY_TWO;
    handle_look("look", "", -1, -1, $command, "", "", $error);
    output_text("\nThe head\'s eyes suddenly open, fixing you in place. " .
                "\"Oh, my,\" he says. \"This is a surprise. I certainly " .
                "never expected to see you in person!\"\n");
    return SUCCESS;
  }

  return CONTINUE;
}

sub desc_for_head
{
  my ($verb, $prep, $dobj, $iobj) = @_;
  my $MUDDY_TWO = 16;
  my $HEAD = 8;
  if ($verb eq "look" && within(PC_OBJ, $MUDDY_TWO) && 
      within($HEAD, $MUDDY_TWO))
  {
    output_text("\nBuried almost to its chin in the mud is " . aname($HEAD) .
                ".\n");
    # fall through to the continue:
  }

  return CONTINUE;
}

sub head_talks_again
{
  my ($verb, $prep, $dobj, $iobj) = @_;
  my $HEAD = 8;
  if (!is_dir($verb) && $verb ne "talk to" && 
      within($HEAD, $state{'objects'}->[PC_OBJ]) && flagged($HEAD))
  {
    output_text("\n\nThe man laughs softly. \"Not long now.\" He grins ".
                "sarcastically. \"An interesting end, don\'t you think? " .
                "Jiminy Cricket never had such an exit scene.\" " .
                ucfirst(thename($HEAD)) . " sinks into the mud and is " .
                "gone.\n");
    $state{'objects'}->[$HEAD] = NOWHERE;

    # fall through to the continue:
  }

  return CONTINUE;
}

sub head_talks
{
  my ($verb, $prep, $dobj, $iobj) = @_;
  my $HEAD = 8;
  if (!is_dir($verb) && $verb ne "talk to" && 
      within($HEAD, $state{'objects'}->[PC_OBJ]) && !flagged($HEAD))
  {
    output_text("\n\n\"Come to gloat?\" the man says. \"Why not? After all, " .
                "you put me here.\"\n");
    flagged($HEAD, 1);
    # fall through to the continue:
  }

  return CONTINUE;
}

sub kick_head
{
  my ($verb, $prep, $dobj, $iobj) = @_;
  my $HEAD = 8;
  if (($verb eq "kick" || $verb eq "attack") && $dobj eq $HEAD)
  {
    output_text(ucfirst(thename($HEAD)) . "'s eyes widen as you draw your " .
                "foot back. \"Terry, no, please, oh God you can't--\" His " .
                "cries are cut short as your foot slams into him. With the " .
                "sound of eggshells cracking, " . thename($HEAD) . 
                " fragments into countless pieces which are quickly lost " .
                "in the mud.\n");
    $state{'objects'}->[$HEAD] = NOWHERE;

    return SUCCESS;
  }

  return CONTINUE;
}

sub kiss_head
{
  my ($verb, $prep, $dobj, $iobj) = @_;
  my $HEAD = 8;
  if ($verb eq "kiss" && $dobj eq $HEAD)
  {
    output_text("The mud coating him makes you think twice.\n");
    return ABORT;
  }

  return CONTINUE;
}

sub take_head
{
  my ($verb, $prep, $dobj, $iobj) = @_;
  my $HEAD = 8;
  if (($verb eq "take" || $verb eq "pull") && $dobj eq $HEAD)
  {
    output_text("You grab hold of his ears, but before you can pull, " .
                thename($HEAD) . " says \"It's no good now. I'm done " .
                "for.\" His eyes narrow at you. \"The least you can do " .
                "is feel guilty.\"\n");
    return ABORT;
  }

  return CONTINUE;
}

sub dig_head
{
  my ($verb, $prep, $dobj, $iobj) = @_;
  my $HEAD = 8;
  if ($verb eq "dig" && $dobj eq $HEAD)
  {
    output_text("You kneel and begin scooping mud from around " .
                thename($HEAD) . " with both hands. As fast as you remove " .
                "the mud, more mud and water return. \"Forget it,\" the " .
                "man says gruffly. \"It's too late now.\" He seems " . 
                "strangely touched by your attempt, however.\n");
    return ABORT;
  }

  return CONTINUE;
}

sub move_in_second_muddy_room_with_head
{
  my ($verb, $prep, $dobj, $iobj, $command, $dobj_str, $iobj_str, $error) = @_;
  my $MUDDY_TWO = 16;
  my $MUDDY_THREE = 17;
  my $HEAD = 8;
  if (is_dir($verb) && within(PC_OBJ, $MUDDY_TWO) && within($HEAD, $MUDDY_TWO))
  {
    output_text("As you walk away, " . thename($HEAD) . " screams at you. " .
                "\"Coward! You put me here! Can't you watch my last " . 
                "minutes?\" Disturbed, you keep walking.\n\n");
    $state{'objects'}->[$HEAD] = NOWHERE;
    $state{'objects'}->[PC_OBJ] = $MUDDY_THREE;
    handle_look("look", "", -1, -1, $command, "", "", $error);
    return SUCCESS;
  }

  return CONTINUE;
}

sub move_in_second_muddy_room
{
  my ($verb, $prep, $dobj, $iobj, $command, $dobj_str, $iobj_str, $error) = @_;
  my $MUDDY_TWO = 16;
  my $MUDDY_THREE = 17;

  if (is_dir($verb) && within(PC_OBJ, $MUDDY_TWO))
  {
    output_text("You strike off in that direction, but soon become " .
                "disoriented by the rain.\n\n");
    $state{'objects'}->[PC_OBJ] = $MUDDY_THREE;
    handle_look("look", "", -1, -1, $command, "", "", $error);
    return SUCCESS;
  }

  return CONTINUE;
}

sub move_in_third_muddy_room
{
  my ($verb, $prep, $dobj, $iobj, $command, $dobj_str, $iobj_str, $error) = @_;
  my $MUDDY_THREE = 17;
  my $MUDDY_TWO = 16;

  if (is_dir($verb) && $verb ne "north" && $verb ne "in" && 
      within(PC_OBJ, $MUDDY_THREE))
  {
    output_text("As you walk away, the building is quickly swallowed by the " .
                "rain and mist.\n\n");
    $state{'objects'}->[PC_OBJ] = $MUDDY_TWO;
    handle_look("look", "", -1, -1, $command, "", "", $error);
    return SUCCESS;
  }

  return CONTINUE;
}

sub enter_muddy_building
{
  my ($verb, $prep, $dobj, $iobj) = @_;
  my $MUDDY_THREE = 17;
  my $MUDDY_TWO = 16;

  if (($verb eq "north" || $verb eq "in") && within(PC_OBJ, $MUDDY_THREE))
  {
    output_text("As you walk through the door of the building the sound " .
                "of the rain stops, as if suddenly turned off.\n\n");
    # fall through to the continue
  }

  return CONTINUE;
}

sub move_in_muddy_building
{
  my ($verb, $prep, $dobj, $iobj) = @_;
  my $MUDDY_BUILDING = 18;

  if (is_dir($verb) && within(PC_OBJ, $MUDDY_BUILDING))
  {
    output_text("You pause to catch your breath for a moment. This " .
                "self-analysis stuff is hard work!\n");
    return ABORT;
  }

  return CONTINUE;
}

sub drink_coke
{
  my ($verb, $prep, $dobj, $iobj) = @_;
  my $COKE = 9;

  if (($verb eq "eat" || $verb eq "open") && $dobj eq $COKE)
  {
    output_text("You crack open " . thename($COKE) . " and take a long, " .
                "slow swig. The healing process has begun.\n");
    msg("win");
    halt();
    return ABORT;
  }

  return CONTINUE;
}

sub wake_dragon
{
  my ($verb, $prep, $dobj, $iobj) = @_;
  my $DRAGON = 1;

  if (($verb eq "wake" || $verb eq "attack") && $dobj eq $DRAGON)
  {
    output_text(ucfirst(thename($DRAGON)) . " wakes!\n");
    msg("die");
    halt();
    return ABORT;
  }

  return CONTINUE;
}

sub dont_disturb_dragon
{
  my ($verb, $prep, $dobj, $iobj) = @_;
  my $DRAGON = 1;

  if (($verb eq "talk to" && $dobj eq $DRAGON) || 
      ($verb eq "give" && $iobj eq $DRAGON))
  {
    output_text("Shh! You don't want to wake " . thename($DRAGON) . ".\n");
    return ABORT;
  }

  return CONTINUE;
}

sub mud_wakes_dragon
{
  my ($verb, $prep, $dobj, $iobj) = @_;
  my $DRAGON = 1;
  my $MUD = 4;

  if (within($DRAGON, $state{'objects'}->[PC_OBJ]) &&
      within($MUD, $state{'objects'}->[PC_OBJ]) &&
      !is_dir($verb) && # give them a break when entering the room
      int(rand(100)) < 50)
  {
    output_text(ucfirst(thename($DRAGON)) . " smells something - awakens - " .
                "attacks you\n");
    msg("die");
    halt();
    return ABORT;
  }

  return CONTINUE;
}

sub bunyon_with_axe
{
  my ($verb, $prep, $dobj, $iobj) = @_;
  my $AXE = 2;
  my $OX = 3;
  my $GROVE = 14;
  if ($verb eq "bunyon" && within($AXE, PC_OBJ))
  {
    output_text("Something you're holding vibrated and...\n");
    $state{'objects'}->[$AXE] = $GROVE;
    $state{'objects'}->[$OX] = $GROVE 
        if within($OX, $state{'objects'}->[PC_OBJ]);

    return SUCCESS;
  }

  return CONTINUE;
}

sub bunyon_otherwise
{
  my ($verb, $prep, $dobj, $iobj) = @_;
  my $AXE = 2;

  if ($verb eq "bunyon")
  {
    output_text("Nothing happens.\n");
    return SUCCESS;
  }

  return CONTINUE;
}

sub swim_in_quicksand_unloaded
{
  my ($verb, $prep, $dobj, $iobj, $command, $dobj_str, $iobj_str, $error) = @_;
  my $QUICKSAND = 12;
  my $LAKE_SHORE = 11;

  if ($verb eq "swim" && within(PC_OBJ, $QUICKSAND) && contents(PC_OBJ) == 0)
  {
    $state{'objects'}->[PC_OBJ] = $LAKE_SHORE;
    handle_look("look", "", -1, -1, $command, "", "", $error);
    return SUCCESS;
  }

  return CONTINUE;
}

sub swim_in_quicksand
{
  my ($verb, $prep, $dobj, $iobj) = @_;
  my $QUICKSAND = 12;

  if ($verb eq "swim" && within(PC_OBJ, $QUICKSAND))
  {
    output_text("Something's too heavy.\n");
    return SUCCESS;
  }

  return CONTINUE;
}

sub climb_tree
{
  my ($verb, $prep, $dobj, $iobj) = @_;
  my $TREE = 5;

  if ($verb eq "climb" && $dobj eq $TREE)
  {
    output_text(ucfirst(thename($TREE)) . " is rotten - chop it down!\n");
    return SUCCESS;
  }

  return CONTINUE;
}

sub tree_blocks_stump
{
  my ($verb, $prep, $dobj, $iobj) = @_;
  my $TREE = 5;
  
  if (($verb eq "in" || $verb eq "down") && 
      within($TREE, $state{'objects'}->[PC_OBJ]))
  {
    output_text(ucfirst(thename($TREE)) . " is in the way!\n");
    return ABORT;
  }

  return CONTINUE;
}

sub chop_tree
{
  my ($verb, $prep, $dobj, $iobj) = @_;
  my $TREE = 5;
  my $AXE = 2;

  if (($verb eq "attack" || $verb eq "chop") && $dobj eq $TREE &&
      (($iobj eq -1 && within($AXE, PC_OBJ)) || ($iobj eq $AXE)))
  {
    output_text("You chop down " . thename($TREE) . ". You can go in the " .
                "stump!\n");
    $state{'objects'}->[$TREE] = NOWHERE;
    return SUCCESS;
  }

  return CONTINUE;
}

sub say_stump
{
  my ($verb, $prep, $dobj, $iobj) = @_;
  my $SWAMP = 13;
  my $TREE = 5;
  if ($verb eq "look" && within(PC_OBJ, $SWAMP) && !within($TREE, $SWAMP))
  {
    output_text("You can go in the stump!\n");
    # fall through to the continue:
  }

  return CONTINUE;
}

sub drop_treasure_here
{
  my ($verb, $prep, $dobj, $iobj) = @_;
  my $IN_STUMP = 15;

  if ($verb eq "drop" && has_trait($dobj, TREASURE) && 
      within(PC_OBJ, $IN_STUMP))
  {
    output_text("More adventure awaits!\n");
    msg("win");
    halt();
    return SUCCESS;
  }

  return CONTINUE;
}

sub display_wumpus
{
  my ($verb, $prep, $dobj, $iobj) = @_;
  my $WUMPUS = 3;
  my $CAVE_ONE = 10;
  if ($verb eq "look" && within($WUMPUS, loc(PC_OBJ)))
  {
    if (within(PC_OBJ, $CAVE_ONE))
    {
      output_text("\nSomething shifts in the darkness ahead, a great silent " .
                  "bulk. Your prey.\n");
    }
    else
    {
      output_text("\n" . ucfirst(thename($WUMPUS)) . " is crouched before " .
                  "you. Blood trickles from the wound your bolt tore, down " .
                  "its hide, puddling on the floor.\n");
    }
    # fall through to the continue:
  }

  return CONTINUE;
}

sub shoot_wumpus_first
{
  my ($verb, $prep, $dobj, $iobj) = @_;
  my $CAVE_ONE = 10;
  my $WUMPUS = 3;

  if ($verb eq "attack" && $dobj eq $WUMPUS && within(PC_OBJ, $CAVE_ONE))
  {
    output_text("Tunk. And you flinch at the hissing shriek that echoes " .
                "down the cave walls. The dark shape retreats, scrambling " .
                "away, as your thudding pulse slows.\n\nMoments later, " .
                "barely sensed, the scent of blood. You have wounded " .
                thename($WUMPUS) . ".\n");
    loc($WUMPUS, NOWHERE);
    return SUCCESS;
  }

  return CONTINUE;
}

BEGIN
{
  my %rel = map { $_ => 1 } qw(forward backward left right up down);
  sub is_relative_dir
  {
    my $dir = shift;
    return exists($rel{$dir}) ? 1 : 0;
  }
}

sub move_past_wumpus
{
  my ($verb, $prep, $dobj, $iobj) = @_;
  my $WUMPUS = 3;

  if (is_relative_dir($verb) && within($WUMPUS, loc(PC_OBJ)))
  {
    output_text("No. Not while " . thename($WUMPUS) . " remains.\n");
    return ABORT;
  }

  return CONTINUE;
}

sub random_cave_desc
{
  my ($verb, $prep, $dobj, $iobj) = @_;
  my $CAVE_TWO = 11;
  my $CAVE_THREE = 12;
  if ($verb eq "look" && (within(PC_OBJ, $CAVE_TWO) || 
                          within(PC_OBJ, $CAVE_THREE)))
  {
    my @pieces = (
      [ "You are in a ", "This is a ", "You look around at a " ],
      [ "cramped roundish room", "small cave", "largish, elongated dome",
        "dull room", "cramped cavern", "smallish roundish cave", 
        "tiny, oblong cavern", ],
      [ ", mazed by small cracks. ", 
        ". A thin layer of mud lies underfoot. ",
        ". The floor is crusted with wet sand. ",
        ". Gravel and small rock chips cover the ground. ",
        ". Ropy pillars rise from floor to roof. ",
        ", walls covered in moss. ",
        ", with a high roof. " ],
      [ "Stalactites hang low overhead, ",
        "Water trickles audibly, somewhere nearby, ",
        "Needles of white calcite spray from the rocks nearby, ",
        "Cascades of white flowstone billow across the walls, ",
        "The uncertain light seems a bit brighter, " ],
      [ "and you can hear water rushing in the distance. ",
        "and the ground is worn into polished flowing curves. ",
        "and a small still pool lies to one side. ",
        "and the darkness presses in on you. " ],
      [ "Passageways lead off in all directions. ",
        "You could head any direction through the cracks in the walls. ",
        "Crevices lead out of the room in many directions. ",
        "Tunnels let you leave the room several ways. ", ],
    );

    output_text("Cavern\n");
    foreach my $p (@pieces)
    {
      output_text($p->[int(rand(@$p))]);
    }
    output_text("\n");

    return SUCCESS;
  }

  return CONTINUE;
}

sub display_blood_trail
{
  my ($verb, $prep, $dobj, $iobj) = @_;
  my $WUMPUS = 3;
  my $CAVE_ONE = 10;
  my $CAVE_TWO = 11;
  my $CAVE_THREE = 12;

  if ($verb eq "look" && !within($WUMPUS, loc(PC_OBJ)) &&
      (within(PC_OBJ, $CAVE_ONE) || within(PC_OBJ, $CAVE_TWO) ||
       within(PC_OBJ, $CAVE_THREE)))
  {
    output_text("\nA trail of dark blood runs across the cavern floor.\n");
    # fall through to the continue:
  }

  return CONTINUE;
}

sub examine_blood
{
  my ($verb, $prep, $dobj, $iobj, $command, $dobj_str) = @_;
  my $WUMPUS = 3;
  my $CAVE_ONE = 10;
  my $CAVE_TWO = 11;
  my $CAVE_THREE = 12;

  if ($verb eq "examine" && $dobj_str =~ /\b(blood|trail)\b/ && 
      !within($WUMPUS, loc(PC_OBJ)) &&
      (within(PC_OBJ, $CAVE_ONE) || within(PC_OBJ, $CAVE_TWO) ||
       within(PC_OBJ, $CAVE_THREE)))
  {
    if (within(PC_OBJ, $CAVE_ONE))
    {
      output_text("The trail of blood leads to the tunnel ahead of you.\n");
    }
    elsif (within(PC_OBJ, $CAVE_TWO))
    {
      output_text("The trail of blood extends from the crack to your left.\n");
    }
    else
    {
      output_text("The trail of blood goes into the passageway behind you. " .
                  "It's very fresh.\n");
    }

    return SUCCESS;
  }

  return CONTINUE;
}

sub move_through_cave
{
  my ($verb, $prep, $dobj, $iobj, $command, $dobj_str, $iobj_str, $error) = @_;
  my $WUMPUS = 3;
  my $CAVE_ONE = 10;
  my $CAVE_TWO = 11;
  my $CAVE_THREE = 12;
  my $CAVE_FOUR = 13;

  if (($verb eq "forward" && within(PC_OBJ, $CAVE_ONE)) ||
      ($verb eq "left" && within(PC_OBJ, $CAVE_TWO)) ||
      ($verb eq "backward" && within(PC_OBJ, $CAVE_THREE)))
  {
    if (within(PC_OBJ, $CAVE_ONE))
    {
      output_text("Slowly first, then faster, you head through the tunnels, " .
                  "following the trail of blood.\n\n");
      loc(PC_OBJ, $CAVE_TWO);
    }
    elsif (within(PC_OBJ, $CAVE_TWO))
    {
      output_text("A few times bats screech nearby as you move through " .
                  "the caverns, but you ignore them, intent on tracking " .
                  "your quarry.\n\n");
      loc(PC_OBJ, $CAVE_THREE);
    }
    else
    {
      output_text("For a time you get stuck, wriggling into a narrow gap " .
                  "in the wall. But eventually you force yourself free and " .
                  "the hunt resumes. Bats scatter before you as you move " .
                  "through the darkness.\n\n");
      loc(PC_OBJ, $CAVE_FOUR);
      loc($WUMPUS, $CAVE_FOUR);
    }

    unread("look");
    return SUCCESS;
  }

  return CONTINUE;
}

sub dont_drop_in_cave
{
  my ($verb, $prep, $dobj, $iobj) = @_;
  my $CAVE_ONE = 10;
  my $CAVE_TWO = 11;
  my $CAVE_THREE = 12;

  if ($verb eq "drop" && 
      (within(PC_OBJ, $CAVE_ONE) || within(PC_OBJ, $CAVE_TWO) ||
       within(PC_OBJ, $CAVE_THREE)))
  {
    output_text("Anything you leave behind would surely be lost.\n");
    return ABORT;
  }

  return CONTINUE;
}

sub need_crossbow_to_attack
{
  my ($verb, $prep, $dobj, $iobj) = @_;
  my $CROSSBOW = 1;

  if ($verb eq "attack" && !within($CROSSBOW, PC_OBJ))
  {
    output_text("Without your crossbow, you are unarmed and unable.\n");
    return ABORT;
  }

  return CONTINUE;
}

sub move_compass_dir
{
  my ($verb, $prep, $dobj, $iobj) = @_;
  my $CAVE_ONE = 10;
  my $CAVE_TWO = 11;
  my $CAVE_THREE = 12;
  my $CAVE_FOUR = 13;

  if ($verb ne "up" && $verb ne "down" && $verb ne "out" && $verb ne "in" &&
      is_dir($verb) && 
      (within(PC_OBJ, $CAVE_ONE) || within(PC_OBJ, $CAVE_TWO) || 
       within(PC_OBJ, $CAVE_THREE) || within(PC_OBJ, $CAVE_FOUR)))
  {
    output_text("Those directions have meaning under the stars and the sun. " .
                "These caves... contain only themselves.\n");
    return ABORT;
  }

  return CONTINUE;
}

sub move_relative_dir
{
  my ($verb, $prep, $dobj, $iobj) = @_;
  my $CAVE_ONE = 10;
  my $CAVE_TWO = 11;
  my $CAVE_THREE = 12;

  if (is_relative_dir($verb) &&
      (within(PC_OBJ, $CAVE_ONE) || within(PC_OBJ, $CAVE_TWO) || 
       within(PC_OBJ, $CAVE_THREE)))
  {
    output_text("You could strike off boldly into the unknown -- but there " .
                "must be a better way to trail the beast.\n");
    return ABORT;
  }

  return CONTINUE;
}

sub shoot_wumpus_second
{
  my ($verb, $prep, $dobj, $iobj) = @_;
  my $CAVE_FOUR = 13;
  my $CROSSBOW = 1;
  my $WUMPUS = 3;

  if ($verb eq "attack" && $dobj eq $WUMPUS && within(PC_OBJ, $CAVE_FOUR))
  {
    output_text("Silently, you slip another bolt into your crossbow and " .
                "cock it. Does it not see you? Does it not care? But you " .
                "intend to finish things here, and slowly raise the bow.\n\n" .
                "Then a screech from behind, and the bats descend like " .
                "autumn storms.\n\nYou pull yourself painfully to your feet " .
                "when consciousness returns, surveying the situation. Your " .
                "crossbow is lost. " . ucfirst(thename($WUMPUS)) . " has " .
                "vanished. And a trail of blood leads upwards, out of the " .
                "caves.\n");
    loc($WUMPUS, NOWHERE);
    loc($CROSSBOW, NOWHERE);
    return SUCCESS;
  }

  return CONTINUE;
}

sub emerge_from_cave
{
  my ($verb, $prep, $dobj, $iobj) = @_;
  my $OUTSIDE_CAVE = 14;

  if (($verb eq "up" || $verb eq "out") && within(PC_OBJ, $OUTSIDE_CAVE))
  {
    output_text("\n\nA trail stretches out across the sands, marked by " .
                "the imprint of sucker feet and a few drops of dried " .
                "blood.\n");
    # fall through to continue
  }

  return CONTINUE;
}

sub ride_camel
{
  my ($verb, $prep, $dobj, $iobj) = @_;
  my $CAMEL = 4;

  if ($verb eq "ride" && $dobj eq $CAMEL)
  {
    output_text("You boost yourself up on the strange beast's back, " .
                "seating yourself by its 'lump' as best you can, then " .
                "kick it into action. It responds -- clearly it is used to " .
                "this journey -- and soon you are burning across the desert " .
                "sands. You keep an eye on the trail: the Wumpus will not " .
                "escape you. Assuming you make it to the oasis in time, " .
                "anyway, and can stay ahead of the pygmies.\n");
    msg("win");
    halt();
    return ABORT;
  }

  return CONTINUE;
}

sub wander_into_anchorhead
{
  my ($verb, $prep, $dobj, $iobj) = @_;
  my $STREET = 13;
  my $KEYS = 9;

  if (($verb eq "north" || $verb eq "south" || $verb eq "northwest") &&
      within(PC_OBJ, $STREET))
  {
    output_text("There's no time to be wandering off around Anchorhead. " .
                "You've got to find " . thename($KEYS) . " and get them to " .
                "Michael -- he's west of here, doing work at the " .
                "university.\n");
    return ABORT;
  }

  return CONTINUE;
}

sub leave_with_keys
{
  my ($verb, $prep, $dobj, $iobj) = @_;
  my $STREET = 13;
  my $KEYS = 9;

  if ($verb eq "west" && within(PC_OBJ, $STREET) && within($KEYS, PC_OBJ))
  {
    output_text("A sullen belch emanates from the clouds, and the rain " .
                "starts coming down harder -- fat, cold drops smacking " .
                "loudly against the cobblestones. Shouldn't it be snowing " .
                "in New England at this time of year? With a sigh, you " .
                "grasp " . thename($KEYS) . " more firmly and head off " .
                "to find Michael.\n");
    msg("win");
    halt();
    return ABORT;
  }

  return CONTINUE;
}

sub leave_without_keys
{
  my ($verb, $prep, $dobj, $iobj) = @_;
  my $STREET = 13;
  my $KEYS = 9;

  if ($verb eq "west" && within(PC_OBJ, $STREET))
  {
    output_text("Much as you'd like to get out of here, there's no point " .
                "in heading back to Michael until you find " . thename($KEYS) .
                ".\n");
    return ABORT;
  }

  return CONTINUE;
}

sub office_is_locked
{
  my ($verb, $prep, $dobj, $iobj) = @_;
  my $STREET = 14;
  my $OFFICE = 17;

  if (($verb eq "east" && within(PC_OBJ, $STREET)) ||
      ($verb eq "west" && within(PC_OBJ, $OFFICE)))
  {
    output_text("The door to the real estate office is locked firmly.\n");
    return ABORT;
  }

  return CONTINUE;
}

sub push_can
{
  my ($verb, $prep, $dobj, $iobj) = @_;
  my $GARBAGE_CAN = 5;

  if (($verb eq "pull" || $verb eq "push" || $verb eq "move") && 
      $dobj eq $GARBAGE_CAN && !flagged($GARBAGE_CAN))
  {
    output_text("Grunting and holding your breath, you manhandle " .
                thename($GARBAGE_CAN) . " under the window.\n");
    flagged($GARBAGE_CAN, 1);
    return SUCCESS;
  }

  return CONTINUE;
}

sub push_can_again
{
  my ($verb, $prep, $dobj, $iobj) = @_;
  my $GARBAGE_CAN = 5;

  if (($verb eq "pull" || $verb eq "push" || $verb eq "move") && 
      $dobj eq $GARBAGE_CAN)
  {
    output_text("You pull " . thename($GARBAGE_CAN) . " away from the " .
                "window.\n");
    flagged($GARBAGE_CAN, 0);
    return SUCCESS;
  }

  return CONTINUE;
}

sub climb_onto_can
{
  my ($verb, $prep, $dobj, $iobj) = @_;
  my $GARBAGE_CAN = 5;

  if ($verb eq "climb" && $dobj eq $GARBAGE_CAN)
  {
    if (flagged($GARBAGE_CAN))
    {
      output_text("You clamber onto " . thename($GARBAGE_CAN) . 
                  ", precariously balanced. You can just reach the lower " .
                  "edge of the window from here.\n");
      flagged(PC_OBJ, 1);
      return SUCCESS;
    }
    else
    {
      output_text("There's no point in climbing on " . thename($GARBAGE_CAN) .
                  " while it's in the middle of the alley.\n");
      return SUCCESS;
    }
  }

  return CONTINUE;
}

sub descend_from_can
{
  my ($verb, $prep, $dobj, $iobj) = @_;
  my $GARBAGE_CAN = 5;

  if (($verb eq "down" || $verb eq "out" || $verb eq "stand" ||
       ($verb eq "get off" && $dobj eq $GARBAGE_CAN)) && flagged(PC_OBJ))
  {
    output_text("Carefully, you descend.\n");
    flagged(PC_OBJ, 0);
    return SUCCESS;
  }

  return CONTINUE;
}

sub implicitly_descend_from_can
{
  my ($verb, $prep, $dobj, $iobj) = @_;
  my $ALLEY = 15;

  if (!within(PC_OBJ, $ALLEY) && flagged(PC_OBJ))
  {
    flagged(PC_OBJ, 0);
    # fall through to the continue
  }

  return CONTINUE;
}

sub enter_office_window_closed
{
  my ($verb, $prep, $dobj, $iobj, $command, $dobj_str, $iobj_str, $error) = @_;
  my $ALLEY = 15;
  my $FILE_ROOM = 16;
  my $WINDOW = 6;

  if (($verb eq "north" || $verb eq "in" || 
       ($verb eq "enter" && $dobj eq $WINDOW) ||
       ($verb eq "enter" && $dobj_str =~ /\boffice\b/)) &&
      within(PC_OBJ, $ALLEY) && flagged(PC_OBJ) && flagged($WINDOW))
  {
    output_text(ucfirst(thename($WINDOW)) . " is closed.\n");
    return ABORT;
  }

  return CONTINUE;
}

sub enter_office_from_can
{
  my ($verb, $prep, $dobj, $iobj, $command, $dobj_str, $iobj_str, $error) = @_;
  my $ALLEY = 15;
  my $FILE_ROOM = 16;
  my $WINDOW = 6;

  if (($verb eq "north" || $verb eq "in" || 
       ($verb eq "enter" && $dobj eq $WINDOW) ||
       ($verb eq "enter" && $dobj_str =~ /\boffice\b/)) &&
      within(PC_OBJ, $ALLEY) && flagged(PC_OBJ))
  {
    output_text("It's a tight squeeze, but you just manage to wriggle " .
                "through, dropping quietly to the floor inside.\n\n");
    $state{'objects'}->[PC_OBJ] = $FILE_ROOM;
    handle_look("look", "", -1, -1, $command, "", "", $error);
    return SUCCESS;
  }

  return CONTINUE;
}

sub enter_office_from_ground
{
  my ($verb, $prep, $dobj, $iobj, $command, $dobj_str, $iobj_str, $error) = @_;
  my $ALLEY = 15;
  my $WINDOW = 6;

  if (($verb eq "north" || $verb eq "in" || 
       ($verb eq "enter" && $dobj eq $WINDOW) ||
       ($verb eq "enter" && $dobj_str =~ /\boffice\b/)) &&
      within(PC_OBJ, $ALLEY))
  {
    output_text("You can't reach the window from the ground.\n");
    return ABORT;
  }

  return CONTINUE;
}

sub disturb_window_from_ground
{
  my ($verb, $prep, $dobj, $iobj) = @_;
  my $WINDOW = 6;

  if ($verb ne "examine" && ($dobj eq $WINDOW || $iobj eq $WINDOW) && 
      !flagged(PC_OBJ))
  {
    output_text("You can't reach the window from the ground.\n");
    return ABORT;
  }

  return CONTINUE;
}

sub play_message
{
  my ($verb, $prep, $dobj, $iobj, $command, $dobj_str, $iobj_str, $error) = @_;
  my $ANSWERING_MACHINE = 8;

  if (($verb eq "push" || $verb eq "play") && 
      $dobj_str =~ /\b(play|button|message)\b/ &&
      within($ANSWERING_MACHINE, loc(PC_OBJ)))
  {
    output_text("For a while there is nothing but a quiet hiss, followed by " .
                "intermittent skirls of strange-sounding static. It sounds " .
                "like one of those annoying glitches where the caller hangs " .
                "up but the machine keeps recording anyway. Then, barely " .
                "audible through the static, you detect what sounds like a " .
                "human voice whispering a single word:\n\n\"Verlac.\"\n\n" .
                "A brief shudder ripples up your back. You remember now, " .
                "\"Verlac\" is the name of this branch of Michael's " .
                "family.\n");
    return SUCCESS;
  }

  return CONTINUE;
}

sub look_up_verlac
{
  my ($verb, $prep, $dobj, $iobj, $command, $dobj_str, $iobj_str, $error) = @_;
  my $FILING_ROOM = 16;
  my $KEYS = 9;

  if (($verb eq "look up" || $verb eq "consult") && 
      (lc($dobj_str) eq "verlac" || lc($iobj_str) eq "verlac") &&
      within(PC_OBJ, $FILING_ROOM))
  {
    output_text("Strange; the file on the Verlac property has been cleaned " .
                "out. Title, deed, all the papers, all of it gone.");
    if (within($KEYS, NOWHERE))
    {
      output_text(" There is, however, " . aname($KEYS) . " tucked down in " .
                  "the hanging folder. You quickly pocket them.\n");
      loc($KEYS, PC_OBJ);
    }
    else
    {
      output_text("\n");
    }

    return SUCCESS;
  }

  return CONTINUE;
}

sub look_up_anything
{
  my ($verb, $prep, $dobj, $iobj) = @_;
  my $FILING_ROOM = 16;

  if (($verb eq "look up" || $verb eq "consult") && 
      within(PC_OBJ, $FILING_ROOM))
  {
    output_text("You don't find anything filed under that heading.\n");
    return SUCCESS;
  }

  return CONTINUE;
}

sub examine_galatea
{
  my ($verb, $prep, $dobj, $iobj, $command, $dobj_str, $iobj_str, $error) = @_;
  my $GALLERY = 18;

  if ($verb eq "examine" && $dobj_str =~ /\b(galatea|statue)\b/ &&
      within(PC_OBJ, $GALLERY))
  {
    output_text("She is facing away from you.  You cannot see her face, " .
                "only her hair, and the line of her shoulder. It's hard to " .
                "know what she's looking at -- the velvet backdrop, if she " .
                "has her eyes open, but there's not much to see in that. " .
                "Mostly, it is obvious, she is not looking at you.\n");
    return SUCCESS;
  }

  return CONTINUE;
}

sub talk_to_galatea_about_cheese
{
  my ($verb, $prep, $dobj, $iobj, $command, $dobj_str, $iobj_str, $error) = @_;
  my $GALLERY = 18;

  if ($verb eq "talk to" && $dobj_str =~ /\b(galatea|statue)\b/ &&
      $iobj_str =~ /\b(cheese|sushi|yams)\b/ && within(PC_OBJ, $GALLERY))
  {
    output_text("Galatea's eyes light up (there must be LEDs in there). " .
                "\"Mm, $iobj_str,\" she says.\n");
    return SUCCESS;
  }

  return CONTINUE;
}

sub talk_to_galatea
{
  my ($verb, $prep, $dobj, $iobj, $command, $dobj_str, $iobj_str, $error) = @_;
  my $GALLERY = 18;

  if ($verb eq "talk to" && $dobj_str =~ /\b(galatea|statue)\b/ &&
      within(PC_OBJ, $GALLERY))
  {
    output_text("Galatea shrugs. \"Sorry, I don't know much about that.\"\n");
    return SUCCESS;
  }

  return CONTINUE;
}

sub make_bed
{
  my ($verb, $prep, $dobj, $iobj, $command, $dobj_str, $iobj_str, $error) = @_;
  my $BEDROOM = 10;

  if ($verb eq "make" && $dobj_str =~ /\bbed\b/ && within(PC_OBJ, $BEDROOM))
  {
    output_text("There's no time to make your bed! You've got IF to write!\n");
    return SUCCESS;
  }

  return CONTINUE;
}

sub watch_tv
{
  my ($verb, $prep, $dobj, $iobj, $command, $dobj_str, $iobj_str, $error) = @_;
  my $TV = 1;

  if ($verb eq "watch" && $dobj eq $TV)
  {
    my @progs = ("a nature documentary", 
                 "a cooking show with a wacky host",
                 "a science-fiction movie about aliens", 
                 "Scarecrow and Mrs. King reruns");

    output_text("You flip through the channels aimlessly, finally settling " .
                "on " . $progs[int(rand(@progs))] . ". But it's not enough " .
                "to inspire you to write your IF game!\n");

    return SUCCESS;
  }

  return CONTINUE;
}

sub eat_cookies
{
  my ($verb, $prep, $dobj, $iobj) = @_;
  my $COOKIES = 4;

  if ($verb eq "eat" && $dobj eq $COOKIES)
  {
    output_text("Mm scrumpf num. You gobble down " . thename($COOKIES) . 
                ", and are just wiping the crumbs off your face when " .
                "suddenly inspiration strikes! That's it! Quickly, to the " .
                "den!\n");
    loc($COOKIES, NOWHERE);
    return SUCCESS;
  }

  return CONTINUE;
}

sub write_if
{
  my ($verb, $prep, $dobj, $iobj, $command, $dobj_str, $iobj_str, $error) = @_;
  my $COMPUTER = 2;
  my $COOKIES = 4;

  if (($verb eq "write" || $verb eq "type" || $verb eq "type on" || 
       $verb eq "use") && $dobj_str =~ /\b(if|computer|game|keyboard)\b/ &&
      within(PC_OBJ, loc($COMPUTER)))
  {
    if (!flagged($COMPUTER))
    {
      output_text("Not with " . thename($COMPUTER) . " off!\n");
      return ABORT;
    }
    elsif (within($COOKIES, NOWHERE))
    {
      output_text("Surfing the crest of a sugar high, you sit down at the " .
                  "computer and are soon pounding out word after word of " .
                  "your IF masterpiece. You're just not sure of the title, " .
                  "but \"Mirrors\" will do for now.\n");
      msg("win");
      halt();
      return ABORT;
    }
    else
    {
      output_text("You sit down at the computer and listlessly type out a " .
                  "few words, then delete them. You need some inspiration, " .
                  "but where can you find such a thing?\n");
      return SUCCESS;
    }
  }

  return CONTINUE;
}

sub display_raymond
{
  my ($verb, $prep, $dobj, $iobj) = @_;
  my $RAYMOND = 5;

  if (($verb eq "look" || is_dir($verb)) && within($RAYMOND, loc(PC_OBJ)))
  {
    if (flagged($RAYMOND))
    {
      output_text("\n" . ucfirst(thename($RAYMOND)) . " is here, " .
                  "white-faced and pasty, staring aimlessly into space. " .
                  "Good.\n");
    }
    else
    {
      output_text("\n" . ucfirst(thename($RAYMOND)) . " is here, spinning " .
                  "around in his swivel chair and making occasional " . 
                  "corrections to his report.\n");
    }

    return SUCCESS;
  }

  return CONTINUE;
}

sub talk_to_raymond
{
  my ($verb, $prep, $dobj, $iobj) = @_;
  my $RAYMOND = 5;
  my $BOSS = 6;

  if ($verb eq "talk to" && $dobj eq $RAYMOND)
  {
    if (flagged($RAYMOND))
    {
      output_text("You grin at " . thename($RAYMOND) . ", showing some " .
                  "teeth. He flinches away. You may be able to make " .
                  "something of this department after all.\n");
    }
    else
    {
      output_text("\"" . ucfirst(thename($RAYMOND)) . ", why did you take " .
                  "my paperclips?\" you inquire, in your calmest " .
                  "voice.\n\n\"Why, I needed to clip some things! It's all " .
                  "for my big report here! " . ucfirst(thename($BOSS)) . 
                  " will be so pleased with me!\" He goes back to working " .
                  "happily, not seeing how little time he has left.\n");
    }

    return SUCCESS;
  }

  return CONTINUE;
}

sub raymond_talk_on_entrance
{
  my ($verb, $prep, $dobj, $iobj) = @_;
  my $RAYMOND = 5;
  my $BOSS = 6;

  if ($verb eq "east" && within($RAYMOND, loc(PC_OBJ)))
  {
    output_text("\n\"Golly, howdy!\" " . thename($RAYMOND) . " says as you " .
                "emerge from your cubicle. \"This report sure is hard work, " .
                "but I'm almost done making the corrections, and then I can " .
                "type them all in and print out the final version. " .
                ucfirst(thename($BOSS)) . " is gonna have his world rocked, " .
                "let me tell ya!\" He spins around his chair from sheer " .
                "glee.\n");
    return SUCCESS; # don't talk any more
  }

  return CONTINUE;
}

sub raymond_babble
{
  my ($verb, $prep, $dobj, $iobj) = @_;
  my $RAYMOND = 5;
  my $REPORT = 7;

  if (within($RAYMOND, loc(PC_OBJ)) && !flagged($RAYMOND) && 
      !within($REPORT, PC_OBJ) && within($REPORT, loc(PC_OBJ)) && 
      int(rand(100)) < 66)
  {
    my @babble = (
      ucfirst(thename($RAYMOND)) . " makes some more marks on his report, " .
        "guffawing to himself.",
      "\"Man, the boss is going to love this!\" " . thename($RAYMOND) . " " .
        "chortles.",
      ucfirst(thename($RAYMOND)) . " spins around in his swivel chair.",
    );
    output_text("\n" . $babble[int(rand(@babble))] . "\n");
    # fall through to the continue
  }

  return CONTINUE;
}

sub pick_up_report_by_raymond
{
  my ($verb, $prep, $dobj, $iobj) = @_;
  my $REPORT = 7;
  my $RAYMOND = 5;

  if ($verb eq "take" && $dobj eq $REPORT && within($RAYMOND, loc(PC_OBJ)) &&
      !flagged($RAYMOND))
  {
    output_text("\"You want to take a gander at that?\" " . thename($RAYMOND) .
                " asks. \"Go for it -- but let me warn ya, it's a real " .
                "hum-dinger!\"\n");
    return CONTINUE;
  }

  return CONTINUE;
}

sub cant_leave_with_report
{
  my ($verb, $prep, $dobj, $iobj) = @_;
  my $REPORT = 7;
  my $RAYMOND = 5;

  if (is_dir($verb) && within($REPORT, PC_OBJ) && 
      within($RAYMOND, loc(PC_OBJ)) && !flagged($RAYMOND))
  {
    output_text("\"Whoah, hey, hold your horses half a minute, brother,\" " . 
                thename($RAYMOND) . " says. \"Don't go making tracks with " .
                "that report of mine until I've typed in the changes!\"" .
                "\n\nYou pause. No, you won't take a stand now. You'll wait " .
                "until you can make a more definitive statement.\n");
    return ABORT;
  }

  return CONTINUE;
}

sub display_mcneeble
{
  my ($verb, $prep, $dobj, $iobj) = @_;
  my $BOSS = 6;

  if (($verb eq "look" || is_dir($verb)) && within($BOSS, loc(PC_OBJ)))
  {
    if (flagged($BOSS))
    {
      output_text("\n" . ucfirst(thename($BOSS)) . " is searching " .
                  "frantically through his filing-cabinets.\n");
    }
    else
    {
      output_text("\n" . ucfirst(thename($BOSS)) . " is here, staring " .
                  "around suspiciously.\n");
    }

    # fall through to the continue
  }

  return CONTINUE;
}

sub talk_to_mcneeble
{
  my ($verb, $prep, $dobj, $iobj) = @_;
  my $RAYMOND = 5;
  my $BOSS = 6;

  if ($verb eq "talk to" && $dobj eq $BOSS)
  {
    if (flagged($BOSS))
    {
      output_text("\"Don't disturb me now! This is a crisis!\" he says, " .
                  "burying his face deeper in his filing-cabinets.\n");
    }
    elsif (flagged($RAYMOND))
    {
      output_text("\"Bad news, sir,\" you say, adopting your gravest tone. " .
                  "\"" . ucfirst(thename($RAYMOND)) . "'s report had a " .
                  "little .. accident.\"\n\n\"What?\" " . thename($BOSS) .
                  " shrieks. \"And the Johnson presentation in twenty " .
                  "minutes! I've got to get something together!\" More " .
                  "agilely than you thought possible, he dives for his " .
                  "filing cabinets and begins rifling through the " .
                  "paperwork.\n");
      flagged($BOSS, 1);
    }
    else
    {
      output_text("\"Sir, I--\" You start to speak, but " . thename($BOSS) .
                  " cuts you off.\n\n\"Not another word -- I know exactly " .
                  "what you're going to say. And let me tell you, I don't " .
                  "care what kind of bigshot you were at your last " .
                  "department, here you're nothing! " . 
                  ucfirst(thename($RAYMOND)) . "'s the new golden boy, and " .
                  "when he finishes that report for the Johnson " .
                  "presentation, I'm going to ride it all the way to the " .
                  "top. Heh heh heh. And where's that depreciation report " .
                  "from you?\"\n");
    }

    return SUCCESS;
  }

  return CONTINUE;
}

sub mcneeble_babble
{
  my ($verb, $prep, $dobj, $iobj) = @_;
  my $BOSS = 6;

  if (within($BOSS, loc(PC_OBJ)) && !flagged($BOSS) && int(rand(100)) < 50)
  {
    my @babble = (
      ucfirst(thename($BOSS)) . " glances warily out at the supply room.",
      ucfirst(thename($BOSS)) . " rubs his palms together and drums his " .
        "fingers on the desk. ",
      "\"I wonder how that report is doing?\" wonders " . thename($BOSS) .
        ".\n",
    );
    output_text("\n" . $babble[int(rand(@babble))] . "\n");
    # fall through to the continue
  }

  return CONTINUE;
}

sub shred_report
{
  my ($verb, $prep, $dobj, $iobj) = @_;
  my $SHREDDER = 8;
  my $REPORT = 7;
  my $RAYMOND = 5;

  if (flagged($SHREDDER) && within($REPORT, $SHREDDER))
  {
    output_text(ucfirst(thename($RAYMOND)) . " looks around. \"Hey, what " .
                "happened to my report?\" Just then, there is a horrific " .
                "grinding noise from " . thename($SHREDDER) . " and bits " .
                "of white confetti emerge from its base. He goes pale as " .
                "white-out.\n\nYou grab his tie and pull him close, so " .
                "he won't miss the message. \"Next time you touch my office " .
                "supplies, it won't just be your report that gets shredded, " .
                "understand?\" " . ucfirst(thename($RAYMOND)) . 
                " leans back again, gasping for air, as you let " .
                thename($SHREDDER) . " drop to the ground. One down, one to " .
                "go.\n");
    flagged($RAYMOND, 1);
    loc($REPORT, NOWHERE);
    loc($SHREDDER, loc(PC_OBJ));

    return SUCCESS;
  }

  return CONTINUE;
}

sub shred_other
{
  my ($verb, $prep, $dobj, $iobj) = @_;
  my $SHREDDER = 8;

  if (flagged($SHREDDER) && contents($SHREDDER) > 0)
  {
    my @c = contents($SHREDDER);

    output_text("With a horrific grinding noise, " . thename($SHREDDER) . 
                " reduces " . thename($c[0]) . " to powder.\n");
    loc($c[0], NOWHERE);

    # fall through to continue
  }

  return CONTINUE;
}

sub mcneeble_blocks_supply
{
  my ($verb, $prep, $dobj, $iobj) = @_;
  my $BOSS = 6;
  my $ABANDONED_CUBE = 17;

  if ($verb eq "east" && within(PC_OBJ, $ABANDONED_CUBE) && !flagged($BOSS))
  {
    output_text("You take a cautious step or two east, only to be brought " .
                "up short by a shout of elderly rage from the office to the " .
                "south. \"Stay away from those supplies!\"\n\nThe wise " .
                "general does not make a direct attack when at a " .
                "disadvantage -- clearly a distraction is what is called " .
                "for.\n");
    return ABORT;
  }

  return CONTINUE;
}

sub cant_reenter_cube
{
  my ($verb, $prep, $dobj, $iobj) = @_;
  my $RAYMONDS_CUBE = 15;
  my $PAPERCLIPS = 9;

  if ($verb eq "west" && within(PC_OBJ, $RAYMONDS_CUBE) && 
      !within($PAPERCLIPS, PC_OBJ))
  {
    output_text("You have sworn an oath not to return with your " .
                "goals unfulfilled.\n");
    return ABORT;
  }

  return CONTINUE;
}

sub return_with_paperclips
{
  my ($verb, $prep, $dobj, $iobj) = @_;
  my $CUBICLE = 14;
  my $PAPERCLIPS = 9;

  if (within(PC_OBJ, $CUBICLE) && within($PAPERCLIPS, PC_OBJ))
  {
    output_text("\nYou slide back smoothly into your seat, and slide a " .
                "paperclip just as smoothly onto the depreciation report. " .
                "Already your mind is planning out the way to your next " .
                "objective: coffee.\n");
    msg("win");
    halt();
    return ABORT;
  }

  return CONTINUE;
}

sub display_swabbies
{
  my ($verb, $prep, $dobj, $iobj) = @_;
  my $SWABBIES = 7;

  if (($verb eq "look" || is_dir($verb)) && within($SWABBIES, loc(PC_OBJ)))
  {
    output_text("\n\nYour crew of swabbies is here, looking around " .
                "semi-alertly.\n");
    # fall through to the continue
  }

  return CONTINUE;
}

sub talk_to_swabbies
{
  my ($verb, $prep, $dobj, $iobj) = @_;
  my $SWABBIES = 7;
  my $SPYGLASS = 8;
  my $ISLAND = 18;
  my $SHOVEL = 9;

  if ($verb eq "talk to" && $dobj eq $SWABBIES)
  {
    if (within($SWABBIES, $ISLAND))
    {
      if (within($SHOVEL, NOWHERE))
      {
        output_text("\"Where's the shovel?\" you demand.\n\"Uh .. Squinteye " .
                    "had it.\"\n\"I never! Squarepeg was supposed to bring " .
                    "it!\"\n\"I told Squashface to take care of it!\"\n" .
                    "\"I got it right here, Cap'n,\" wheezes one, finally." .
                    "\n\"Thank you, Squeezebox,\" you say, taking the " .
                    "shovel.\n");
        loc($SHOVEL, PC_OBJ);
      }
      else
      {
        output_text("\"Well, you swabbies, get to work burying it!\"\n\n" .
                    "There is a pause. \"Sorry, Cap'n. Union regs say the " .
                    "captain has to bury the treasure.\"\n");
      }
    }
    elsif (flagged($SPYGLASS))
    {
      if (flagged($SWABBIES))
      {
        output_text("\"We're here, Cap'n,\" says First Mate Squarepeg. " .
                    "\"If ye go ashore I\'ll tell the men to follow.\"\n");
      }
      else
      {
        output_text("\"Turn the ship south-south-east!\" you call. \"Land " .
                    "ho!\"\n\nThe crew pulls lustily on the rigging and " .
                    "shortly thereafter the island looms to the north.\n");
        flagged($SWABBIES, 1);
      }
    }
    else
    {
      output_text("\"Where do you want us to sail to, Cap'n?\" asks " .
                  "Second Mate Squinteye. \"We ain't seen land in days.\"\n");
    }

    return SUCCESS;
  }

  return CONTINUE;
}

sub keelhaul_swabbies
{
  my ($verb, $prep, $dobj, $iobj) = @_;
  my $SWABBIES = 7;

  if ($verb eq "keelhaul" && $dobj eq $SWABBIES)
  {
    output_text("*SPLOOSH* Nothing like a bit of keelhauling for showing " .
                aname($SWABBIES) . " who's boss.\n");
    return SUCCESS;
  }

  return CONTINUE;
}

sub leave_undocked_ship
{
  my ($verb, $prep, $dobj, $iobj) = @_;
  my $SWABBIES = 7;

  if ($verb eq "north" && !flagged($SWABBIES))
  {
    output_text("Safety rules prohibit leaving the ship while on the " .
                "high seas.\n");
    return ABORT;
  }

  return CONTINUE;
}

sub swabbies_leave_docked_ship
{
  my ($verb, $prep, $dobj, $iobj) = @_;
  my $SWABBIES = 7;
  my $ISLAND = 18;

  if (within(PC_OBJ, $ISLAND) && !within($SWABBIES, $ISLAND))
  {
    output_text("\n\nYour crew swarms out of the ship and onto the beach, " .
                "immediately beginning an imprompt game of freeze tag.\n");
    loc($SWABBIES, $ISLAND);
    return CONTINUE;
  }

  return CONTINUE;
}

sub look_through_spyglass
{
  my ($verb, $prep, $dobj, $iobj) = @_;
  my $CROWS_NEST = 17;
  my $SPYGLASS = 8;
  
  if ($verb eq "look through" && $dobj eq $SPYGLASS)
  {
    if (!within(PC_OBJ, $CROWS_NEST))
    {
      output_text("You can't see anything from down here!\n");
      return SUCCESS;
    }
    elsif (flagged($SPYGLASS))
    {
      output_text("You've already located the island -- you just need to " .
                  "tell your crew about it.\n");
      return ABORT;
    }
    else
    {
      output_text("You scan the waters through the spyglass. For a while " .
                  "you see nothing, but then you catch a distance glimpse " .
                  "of a desert isle. The perfect place!\n");
      flagged($SPYGLASS, 1);
      return SUCCESS;
    }
  }

  return CONTINUE;
}

sub count_treasure
{
  my ($verb, $prep, $dobj, $iobj) = @_;
  my $TREASURE = 6;

  if ($verb eq "count" && $dobj eq $TREASURE)
  {
    output_text("Your wealth is exactly 69,105 gold coins. Give or take.\n");
    return SUCCESS;
  }

  return CONTINUE;
}

sub bury_treasure
{
  my ($verb, $prep, $dobj, $iobj) = @_;
  my $TREASURE = 6;
  my $SHOVEL = 9;

  if (($verb eq "bury" || $verb eq "hide") && $dobj eq $TREASURE)
  {
    if (within($SHOVEL, PC_OBJ))
    {
      output_text("Working quickly, you dig a big hole and lower the " .
                  "treasure to the bottom, then fill the hole in. With a " .
                  "spring in your pegleg and a shanty on your lips, you " .
                  "and your crew return to the ship and sail off into " .
                  "the sunset.\n");
      msg("win");
      halt();
      return ABORT;
    }
    else
    {
      output_text("Uh-oh -- where's the shovel?\n");
      return ABORT;
    }
  }

  return CONTINUE;
}

sub leave_alley
{
  my ($verb, $prep, $dobj, $iobj) = @_;
  my $ALLEY = 15;
  my $COMPUTER = 9;

  if ($verb eq "north" && within(PC_OBJ, $ALLEY))
  {
    if (flagged($COMPUTER))
    {
      output_text("You slip out of the alley and off to meet your contact.\n");
      msg("win");
      halt();
      return ABORT;
    }
    else
    {
      output_text("The mission isn't over.\n");
      return ABORT;
    }
  }

  return CONTINUE;
}

sub door_blocks_storage
{
  my ($verb, $prep, $dobj, $iobj) = @_;
  my $ALLEY = 15;
  my $DOOR = 6;

  if ($verb eq "south" && flagged($DOOR) && within(PC_OBJ, $ALLEY))
  {
    output_text(ucfirst(thename($DOOR)) . " is closed and locked.\n");
    return ABORT;
  }

  return CONTINUE;
}

sub morph_into_key
{
  my ($verb, $prep, $dobj, $iobj) = @_;
  my $DEVICE = 5;

  if (($verb eq "unlock" || $verb eq "lock") && $iobj eq $DEVICE)
  {
    output_text(ucfirst(thename($DEVICE)) . " whirrs quietly, extending " .
                "a pair of thin picklocks.\n\n");
    # fall through to the continue
  }

  return CONTINUE;
}

sub morph_into_light
{
  my ($verb, $prep, $dobj, $iobj) = @_;
  my $DEVICE = 5;

  if (($verb eq "turn on" || $verb eq "turn off" || $verb eq "switch") && 
      $dobj eq $DEVICE)
  {
    output_text("A panel on " . thename($DEVICE) . " slides open to expose " .
                "a switch and a shaded bulb.\n\n");
    # fall through to the continue
  }

  return CONTINUE;
}

sub disable_camera
{
  my ($verb, $prep, $dobj, $iobj) = @_;
  my $DEVICE = 5;
  my $CAMERA = 8;

  if (($verb eq "point" || $verb eq "aim" || $verb eq "attack" || 
       $verb eq "shoot") &&
      (($dobj eq $DEVICE && $iobj eq $CAMERA) || 
       ($iobj eq $DEVICE && $dobj eq $CAMERA)))
  {
    if (flagged($CAMERA))
    {
      output_text(ucfirst(thename($CAMERA)) . " is already disabled.\n");
      return ABORT;
    }
    else
    {
      output_text("There is a quiet vibration from " . thename($DEVICE) . 
                  ". The red light on the camera fades and blinks " .
                  "out. That should last long enough if you move quickly.\n");
      flagged($CAMERA, 1);
      return SUCCESS;
    }
  }

  return CONTINUE;
}

sub disable_other_mechanism
{
  my ($verb, $prep, $dobj, $iobj, $dobj_words, $iobj_words) = @_;
  my $DEVICE = 5;
  my $COMPUTER = 9;

  if (($verb eq "point" || $verb eq "aim" || $verb eq "attack" || 
       $verb eq "shoot") &&
      (($dobj eq $DEVICE && $iobj eq $COMPUTER) || 
       ($iobj eq $DEVICE && $dobj eq $COMPUTER) ||
       ($dobj eq $DEVICE && $iobj_words =~ /\breader\b/) ||
       ($iobj eq $DEVICE && $dobj_words =~ /\breader\b/)))
  {
    output_text("You need that in working order to proceed.\n");
    return ABORT;
  }

  return CONTINUE;
}

sub cant_affect_camera
{
  my ($verb, $prep, $dobj, $iobj) = @_;
  my $CAMERA = 8;

  if ($verb ne "examine" && ($dobj eq $CAMERA || $iobj eq $CAMERA))
  {
    output_text(ucfirst(thename($CAMERA)) . " is out of reach.\n");
    return ABORT;
  }

  return CONTINUE;
}

sub camera_watches_swipe
{
  my ($verb, $prep, $dobj, $iobj) = @_;
  my $CAMERA = 8;

  if ($verb eq "swipe" && !flagged($CAMERA))
  {
    output_text("There's no way with " . thename($CAMERA) . " watching.\n");
    return ABORT;
  }

  return CONTINUE;
}

sub examine_swipe_reader
{
  my ($verb, $prep, $dobj, $iobj, $command, $dobj_words) = @_;
  my $WAREHOUSE = 17;
  if ($verb eq "examine" && $dobj_words =~ /\breader\b/ &&
      within(PC_OBJ, $WAREHOUSE))
  {
    output_text("The swipecard reader is built discreetly into the far " .
                "wall. It looks as though you can just swipe a card in it.\n");
    return SUCCESS;
  }

  return CONTINUE;
}

sub put_to_swipe
{
  my ($verb, $prep, $dobj, $iobj, $command, $dobj_words, $iobj_words) = @_;
  my $WAREHOUSE = 17;
  if ($verb eq "put" && $iobj_words =~ /\breader\b/ &&
      within(PC_OBJ, $WAREHOUSE))
  {
    unread("swipe " . $dobj_words);
    return ABORT;
  }

  return CONTINUE;
}

sub swipe_card
{
  my ($verb, $prep, $dobj, $iobj) = @_;
  my $WAREHOUSE = 17;
  my $ID_CARD = 7;

  if ($verb eq "swipe" && $dobj eq $ID_CARD && within(PC_OBJ, $WAREHOUSE))
  {
    if (flagged($ID_CARD))
    {
      output_text("The trapdoor closes silently.\n");
      flagged($ID_CARD, 0);
    }
    else
    {
      output_text("A trapdoor in the floor silently slides open.\n");
      flagged($ID_CARD, 1);
    }
    return ABORT;
  }

  return CONTINUE;
}

sub swipe_multi
{
  my ($verb, $prep, $dobj, $iobj) = @_;
  my $WAREHOUSE = 17;
  my $DEVICE = 5;

  if ($verb eq "swipe" && $dobj eq $DEVICE && within(PC_OBJ, $WAREHOUSE))
  {
    output_text(ucfirst(thename($DEVICE)) . " emits a faint buzz. It must " .
                "not be able to simulate the passcode.\n");
    return SUCCESS;
  }

  return CONTINUE;
}

sub display_trapdoor
{
  my ($verb, $prep, $dobj, $iobj) = @_;
  my $WAREHOUSE = 17;
  my $ID_CARD = 7;

  if (($verb eq "look" || is_dir($verb)) && within(PC_OBJ, $WAREHOUSE) &&
      flagged($ID_CARD))
  {
    output_text("\nA trapdoor leads downwards.\n");
    # fall through to the continue
  }

  return CONTINUE;
}

sub trapdoor_blocks_down
{
  my ($verb, $prep, $dobj, $iobj) = @_;
  my $WAREHOUSE = 17;
  my $ID_CARD = 7;
  if ($verb eq "down" && within(PC_OBJ, $WAREHOUSE) && !flagged($ID_CARD))
  {
    output_text("How?\n");
    return ABORT;
  }

  return CONTINUE;
}

sub plug_in_computer
{
  my ($verb, $prep, $dobj, $iobj) = @_;
  my $DEVICE = 5;
  my $COMPUTER = 9;
  if (($verb eq "plug" || $verb eq "plug in") && $dobj eq $DEVICE &&
      within($COMPUTER, loc(PC_OBJ)))
  {
    if (flagged($COMPUTER))
    {
      output_text("You've already got the data.\n");
      return ABORT;
    }
    else
    {
      output_text("The device bleeps quietly as it breaks the encryption " .
                  "on the data and transfers a copy to its internal " .
                  "storage. It bleeps again when finished, and you detach " .
                  "it from the computer.\n");
      flagged($COMPUTER, 1);
      return SUCCESS;
    }
  }

  return CONTINUE;
}

sub close_trapdoor
{
  my ($verb, $prep, $dobj, $iobj) = @_;
  my $WAREHOUSE = 17;
  my $ID_CARD = 7;

  if (($verb eq "up" || $verb eq "wait") && within(PC_OBJ, $WAREHOUSE) &&
      flagged($ID_CARD))
  {
    output_text("\nThe trapdoor slides shut.\n");
    flagged($ID_CARD, 0);
    # fall through to the continue
  }

  return CONTINUE;
}

1;
